--- 悬浮球
---date:2018-06-22
---author: hexujun@cmcm.com

local FLOAT_TAG = "float_window";
local floatBall;
local FLOAT_BALL_POSITION = { anchor = Anchor().TOPRIGHT, x = -449, y = 129 }
local settingView
local isFloatWindowShowing;
local gbSettingViewOpen = false
local skinCount;

local function printLog(msg)
    commonlib_print(FLOAT_TAG, msg)
end

local function setSettingViewIcon()
    commonlib_setFloatBallIcon("common://image/icon_floatball.png")
    commonlib_setSettingViewIcon("common://image/icon_settingview.png")
    commonlib_setSettingViewIconSize(97.67, 65)
    commonlib_setSettingViewIconMargin(12, 0, 0, 0)
    commonlib_setSettingViewIconExtrudeHeight(21)
    commonlib_setFloatBallDragAndClick(true)
end


local function showFloatBall(show)
    printLog("showFloatBall " .. tostring(show))
    if floatBall == nil then
        printLog("showFloatBall return")
        return
    end

    commonlib_runOnUIThread(
        function()
            if floatBall == nil then
                printLog("showFloatBall return")
                return
            end

            if gbSettingViewOpen then
                printLog("showFloatBall return, setting view is opened")
                return
            end

            isFloatWindowShowing = show;

            if show == true then
                printLog("showFloatBall show")
                floatBall:show()
            else
                printLog("showFloatBall hide")
                floatBall:hide()
            end
        end);
end


local function moveFloatBall()
    commonlib_runOnUIThread(function()
        if floatBall == nil then
            return
        end
        floatBall:moveCenterTo(FLOAT_BALL_POSITION.anchor, FLOAT_BALL_POSITION.x, FLOAT_BALL_POSITION.y)
    end);
end

local function initVariable()
    printLog("initVariable:skinCount"..commonlib_toString(skinCount))

    floatBall = commonlib_getFloatBall();
    settingView = commonlib_getSettingView();
    floatBall:setTargetPackage(commonlib_getGameId())
    floatBall:setOnClickListener(function()
        printLog("悬浮球被点击");
        if common_skin_getSkinCount() > 0 then
            settingView:show();
            floatBall:hide();
            isFloatWindowShowing = false;
        end
    end);
    isFloatWindowShowing = false;
    moveFloatBall();
    printLog("initVariable:end")
end



local function unInitVariable()
    floatBall = nil
    isFloatWindowShowing = false;
end

local function onSettingViewOpen()
    gbSettingViewOpen = true
    printLog("onSettingViewOpen")
end
local function onSettingViewClose()
    gbSettingViewOpen = false
    printLog("onSettingViewClose")
end

local function work()
    printLog("work:start:gbSettingViewOpen,floatBallShowing:"
            ..commonlib_toString(gbSettingViewOpen)..","..commonlib_toString(isFloatWindowShowing))
    if isFloatWindowShowing == true or gbSettingViewOpen == true then
        printLog("work:end 1");
        return
    end

    commonlib_runOnUIThread(function()
        showFloatBall(true);
    end)
    printLog("work:end 2");
end

local function registerGameEvents()
    commonlib_registerCallback(common_game_events_callbacks[KEY_GAME_WORK], work)
    commonlib_registerSettingViewOpenCallback(onSettingViewOpen)
    commonlib_registerSettingViewCloseCallback(onSettingViewClose)
end

local function unregisterGameEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_GAME_WORK], work)

    commonlib_unRegisterSettingViewOpenCallback(onSettingViewOpen)
    commonlib_unRegisterSettingViewCloseCallback(onSettingViewClose)
end

local function init()
    printLog("init begin")
    registerGameEvents()
    initVariable()

    -- 开始设置悬浮球的外观
    setSettingViewIcon()
end

local function uninit()
    printLog("uninit")
    unInitVariable()
    unregisterGameEvents()
end

table.insert(common_life_circle_callbacks, { init, uninit})

