local TAG = "cldts_skin"

local SKIN_IMG_DIR = "img/skin/default/";

local KEY_IS_SWITCH_CHECKBOX_SELECTED = "is_hyxd_switch_checkbox_selected"

local SKIN_ID_TABLE = {
    LEFT_FIST = "0",
    LEFT_GUN = "2",
    RIGHT_FIST = "1",
    RIGHT_GUN = "3"
}

local SKIN_TAB_PAGE_NAME = "皮肤美化";
local SKIN_SWITCH_CHECKBOX_TEXT = "开启皮肤美化"
local THEMES_1080P = {
    ["default"] = {
        ["widget"] = {
            ["0"] = { "left_fist.png", Anchor().BOTTOMLEFT, 11, -564, 157, 158 },
            ["1"] = { "right.png", Anchor().BOTTOMRIGHT, -371, -356, 184, 184 },
            ["2"] = { "left_fist.png", Anchor().BOTTOMLEFT, 11, -564, 157, 158 },
            ["3"] = { "right.png", Anchor().BOTTOMRIGHT, -371, -353, 184, 184 }
        }
    }
}

local THEMES_2160P = {
    ["default"] = {
        ["widget"] = {
            ["0"] = { "left_fist.png", Anchor().BOTTOMLEFT, 24, -563, 157, 158 },
            ["1"] = { "right.png", Anchor().BOTTOMRIGHT, -418, -356, 184, 184 },
            ["2"] = { "left_fist.png", Anchor().BOTTOMLEFT, 24, -563, 157, 158 },
            ["3"] = { "right.png", Anchor().BOTTOMRIGHT, -418, -356, 184, 184 }
        }
    }
}

local skinsTable = {};
local skinFloatView = UnClickableFloatView();
local cbSwitch = nil;
local isCbSwitchSelected = true;
local themes = nil
local hasReportShowSkin = false;

local function printLog(msg)
    commonlib_print(TAG, msg)
end

local function createSkins()
    local skins = themes["default"]["widget"];
    for skinId, skinValue in pairs(skins) do
        local imageSrc = SKIN_IMG_DIR .. skinValue[1];
        local skinImage = skinFloatView:addImageView(imageSrc, skinValue[2], skinValue[3], skinValue[4], skinValue[5], skinValue[6], ZOrder().zorder_bottom);
        skinsTable[skinId] = { skinView = skinImage, isShowing = true, shouldShow = true }
    end
end

local function destroySkins()
    for index, skinInfo in pairs(skinsTable) do
        skinInfo.skinView:destroy();
    end
end

local function reportHyxdShowSkin(reason)
    if not hasReportShowSkin then
        commonlib_report(GAMEMASTER_FUNCTION_DISABLE_REASON_TABLE,
            {
                [GAMEMASTER_FUNCTION_DISABLE_REASON_STEP] = STEP_SHOW_SKIN,
                [GAMEMASTER_FUNCTION_DISABLE_REASON_GAMENAME] = HYXD_APP_NAME,
                [GAMEMASTER_FUNCTION_DISABLE_REASON] = reason
            });
        hasReportShowSkin = true;
    end
end

local function updateSkins()
    for index, skinInfo in pairs(skinsTable) do
        printLog("遍历皮肤id ： " .. tostring(index))
        printLog("皮肤当前状态：" .. tostring(skinInfo.isShowing))
        printLog("皮肤应当处于的状态：" .. tostring(skinInfo.shouldShow))

        if skinInfo.isShowing ~= skinInfo.shouldShow then
            if skinInfo.shouldShow then
                printLog("开始显示皮肤")
                skinInfo.skinView:show();
                reportHyxdShowSkin(REASON_SUCCEED);
            else
                printLog("开始隐藏皮肤")
                skinInfo.skinView:hide();
            end
            skinInfo.isShowing = skinInfo.shouldShow;
        end
    end
end

local function showAllSkins()
    printLog("开始显示所有皮肤")
    for index, skinInfo in pairs(skinsTable) do
        skinInfo.shouldShow = true;
    end
    updateSkins();
end

local function hideAllSkins()
    printLog("开始隐藏所有皮肤")
    for index, skinInfo in pairs(skinsTable) do
        skinInfo.shouldShow = false;
    end
    updateSkins();
end

local function onSkinCheckboxSelected(checkBoxId, isChecked)
    isCbSwitchSelected = isChecked
    commonlib_setSharedPrefBoolean(KEY_IS_SWITCH_CHECKBOX_SELECTED, isChecked)
    onSkinBeautifySettingsChange2(isChecked)
end

local function initSettingUI()
    printLog(" initSettingUI ")

    local settingView = SettingView();
    local tabPage = settingView:addTableView(SKIN_TAB_PAGE_NAME);

    local Param = create("RelativeLayoutParam");
    local cbParamProperty = {
        layout_marginTop = "20dp",
        layout_marginLeft = "25dp"
    };
    Param:setLayoutParam(cbParamProperty);
    isCbSwitchSelected = commonlib_getSharedPrefBoolean(KEY_IS_SWITCH_CHECKBOX_SELECTED, true)
    cbSwitch = tabPage:addCheckBoxView(SKIN_SWITCH_CHECKBOX_TEXT, isCbSwitchSelected, Param, onSkinCheckboxSelected);
end

local function setSkinState(id, shouldShow)
    printLog("开始设置皮肤id ：" .. tostring(id) .. " : " .. tostring(shouldShow))
    if skinsTable[id] ~= nil then
        skinsTable[id].shouldShow = shouldShow
    end
end

local function onBackpackIconChange(hasIcon)
    printLog("onBackpackIconChange : " .. tostring(hasIcon))
    if not isCbSwitchSelected then
        printLog("Checkbox 未选中，不做任何操作")
        return
    end
    if not hasIcon then
        commonlib_runOnUIThread(hideAllSkins)
    end
end

local function onRightGunIconChange(hasIcon)
    printLog("onRightGunIconChange : " .. tostring(hasIcon))
    if not isCbSwitchSelected then
        printLog("Checkbox 未选中，不做任何操作")
        return
    end
    setSkinState("1", hasIcon)
    --setSkinState("3", not hasIcon)
    commonlib_runOnUIThread(updateSkins)
end

local function onRightFistIconChange(hasIcon)
    printLog("onRightFistIconChange : " .. tostring(hasIcon))
    if not isCbSwitchSelected then
        printLog("Checkbox 未选中，不做任何操作")
        return
    end
    --setSkinState("1", not hasIcon)
    setSkinState("3", hasIcon)
    commonlib_runOnUIThread(updateSkins)
end

local function onLeftGunIconChange(hasIcon)
    printLog("onLeftGunIconChange : " .. tostring(hasIcon))
    if not isCbSwitchSelected then
        printLog("Checkbox 未选中，不做任何操作")
        return
    end
    setSkinState("0", hasIcon)
    --setSkinState("2", not hasIcon)
    commonlib_runOnUIThread(updateSkins)
end

local function onLeftFistIconChange(hasIcon)
    printLog("onLeftFistIconChange : " .. tostring(hasIcon))
    if not isCbSwitchSelected then
        printLog("Checkbox 未选中，不做任何操作")
        return
    end
    --setSkinState("0", not hasIcon)
    setSkinState("2", hasIcon)
    commonlib_runOnUIThread(updateSkins)
end

local function onSettingViewClose()
    printLog("设置界面被关闭")
    if not commonlib_getGameParam(KEY_BACKPACK) or not isCbSwitchSelected then
        hideAllSkins()
    else
        setSkinState(SKIN_ID_TABLE.LEFT_FIST, commonlib_getGameParam(KEY_LEFT_FIST))
        setSkinState(SKIN_ID_TABLE.LEFT_GUN, commonlib_getGameParam(KEY_LEFT_GUN))
        setSkinState(SKIN_ID_TABLE.RIGHT_FIST, commonlib_getGameParam(KEY_RIGHT_FIST))
        setSkinState(SKIN_ID_TABLE.RIGHT_GUN, commonlib_getGameParam(KEY_RIGHT_GUN))
        updateSkins()
    end
end

local function registerEvents()
    printLog("开始注册时间")
    printLog("common_game_events_callbacks : " .. tostring(common_game_events_callbacks))

    commonlib_registerCallback(common_game_events_callbacks[KEY_BACKPACK], onBackpackIconChange)
    commonlib_registerCallback(common_game_events_callbacks[KEY_RIGHT_GUN], onRightGunIconChange)
    commonlib_registerCallback(common_game_events_callbacks[KEY_RIGHT_FIST], onRightFistIconChange)
    commonlib_registerCallback(common_game_events_callbacks[KEY_LEFT_GUN], onLeftGunIconChange)
    commonlib_registerCallback(common_game_events_callbacks[KEY_LEFT_FIST], onLeftFistIconChange)
    commonlib_registerSettingViewCloseCallback(onSettingViewClose)
end

local function unRegisterEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_BACKPACK], onBackpackIconChange)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_RIGHT_GUN], onRightGunIconChange)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_RIGHT_FIST], onRightFistIconChange)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_LEFT_GUN], onLeftGunIconChange)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_LEFT_FIST], onLeftFistIconChange)
    commonlib_unRegisterSettingViewCloseCallback(onSettingViewClose)
end

local function init()
    printLog("skin script init")

    if commonlib_isScreenRatio18To9() then
        themes = THEMES_2160P
    else
        themes = THEMES_1080P
    end

    registerEvents();

    initSettingUI();

    createSkins();

    hideAllSkins();
end

local function uninit()
    printLog("skin script uninit")
    destroySkins();
    unRegisterEvents();
    hasReportShowSkin = false;
end

table.insert(common_life_circle_callbacks, { init, uninit });