
local function printLog(msg)
    commonlib_print("common_state", msg)
end

local INIT_INDEX = 1
local UNINIT_INDEX = 2

--- ******游戏运行状态相关******----
HLDDZ_KEY_GAME_STATE = "gameState";
HLDDZ_GAMESTATE_IN_HALL = -3
HLDDZ_GAMESTATE_UNKNOW = -2; --未知
HLDDZ_GAMESTATE_IN_GAMESTART = -1;--游戏开始
HLDDZ_GAMESTATE_LOADING = 0;--游戏加载状态，识别模式
HLDDZ_GAMESTATE_RUNNING = 1;--游戏进行中
HLDDZ_GAMESTATE_OVER = 2;--结束游戏

HLDDZ_KEY_IN_GAME_HALL = "HlddzInGameHall"--游戏大厅
HLDDZ_KEY_IN_GAME_START = "HlddzInGameHome"--START页
HLDDZ_KEY_ENTER_START = "HlddzKeyEnterHome" --进入游戏START页
HLDDZ_KEY_ENTER_GAME = "HlddzEnterGame"--进入游戏
HLDDZ_KEY_GAME_RUNNING = "HlddzGameRunning"--游戏运行中
HLDDZ_KEY_GAME_OVER = "HlddzGameOver"--游戏结束
HLDDZ_KEY_GAME_OVER_PAGE = "HlddzGameOverPage"--在游戏结页面
HLDDZ_KEY_GAME_STATUS_CHANGED = "HlddzGameStatusChanged"--游戏状态变更


HLDDZ_KEY_SWITCH_FLOAT_BALL_SHOW_OR_HID ="hlddz_key_switch_float_ball_show_or_hid"
--欢乐斗地主功能总开关
HLDDZ_FUNC_KEY_ENABLE_KEY = "hlddz_func_key_enable_key"

HLDDZ_KEY_GAME_RUNNING_COVER_STATUS = "HlddzRuningCoverStatus"
--- 遮挡类型
HLDDZ_GAME_COVER_NONE = 0;    -- 无遮挡
HLDDZ_GAME_COVER_UNKNOWN = 1; -- 未知遮挡

-- 用于回调到各个业务脚本的回调函数表
common_game_events_callbacks = {
    [HLDDZ_KEY_ENTER_GAME] = {},
    [HLDDZ_KEY_GAME_RUNNING] = {},
    [HLDDZ_KEY_GAME_OVER] = {},
    [HLDDZ_KEY_IN_GAME_START] = {},
    [HLDDZ_KEY_ENTER_START] = {},
    [HLDDZ_KEY_GAME_STATUS_CHANGED]={},
    [HLDDZ_KEY_GAME_OVER_PAGE] = {},
    [HLDDZ_KEY_IN_GAME_HALL] = {},
}

--- GAME_MODE
HLDDZ_KEY_GAME_MODE = "HlddzGameMode"
HLDDZ_GAME_MODE_UNKNOW = 0 --
HLDDZ_GAME_MODE_NORMAL = 1
HLDDZ_GAME_MODE_LAIZI = 2
HLDDZ_GAME_MODE_THREE_DISCS = 3

-- PICK POKER
HLDDZ_KEY_POKER_NUMBER_TYPE = "HlddzPokerNumberType"

HLDDZ_KEY_GAME_OVER_SUCCESS = "HlddzGameOverSuceess"

local CAFFA2_GAME_STATE = {
    cd_build_game_start_left = {{index = 7, prob = 0.88}, {index = 8, prob = 0.88}, {index = 9, prob = 0.70}},
    cd_build_game_start_right =  {{index = 7, prob = 0.88}, {index = 8, prob = 0.88}, {index = 9, prob = 0.70}},
    cd_build_game_over_failed =  {{index = 15, prob = 0.92}},
    cd_build_game_over_success =  {{index = 15, prob = 0.92}},
    cd_build_game_run_wait_right = {{index = 10, prob = 0.85}, {index = 11, prob = 0.82}, {index = 12, prob = 0.8}, {index = 13, prob = 0.82}, {index = 14, prob = 0.8}},
    cd_build_game_run_wait_left = {{index = 10, prob = 0.85}, {index = 11, prob = 0.82}, {index = 12, prob = 0.8}, {index = 13, prob = 0.82}, {index = 14, prob = 0.8}},
}

local isCanTracePerformance
local performanceTrace
local gNewGameEngine;
local isStart = nil
local isMatchRun = false
local reportWaitStartReport = false

local funcCloudSwitch = nil


local function getHlddzFuncCloudSwitch()
    printLog("getHlddzFuncCloudSwitch ")
    if funcCloudSwitch ~= nil then
        return funcCloudSwitch
    end
    funcCloudSwitch = commonlib_getUtilTool():getHlddzFuncCloudSwitch(HLDDZ_FUNC_KEY_ENABLE_KEY, true)
    return funcCloudSwitch
end

function matchCaffe2Classify(matchString)
    local index = 0
    local prob = 0
    --local time1 = commonlib_getCurrentTime()
    local t = gNewGameEngine:matchConditionOnTable(matchString)
    --local time2 = commonlib_getCurrentTime()
    --commonlib_print("matchCaffe2Classify time=", tostring(time2 - time1))
    if t == nil then
        printLog("matchConditionOnTable(" .. matchString .. ")=nil")
        return false
    end

    --commonlib_print(matchString .. ", t=" .. tostring(t) .. ", size=" .. tostring(#t))
    for key, value in pairs(t) do
        if value.prob and value.prob > prob then
            prob = value.prob
            if value.index then
                index = value.index
            end
        end
    end

    for k, v in pairs(CAFFA2_GAME_STATE[matchString]) do
        if v.index == index and prob >= v.prob then
            return true
        end
    end

    return false
end

local function setCurrentGameCoverType(currentCoverType, lastCoverType)
    if lastCoverType ~= currentCoverType then
        printLog("COVER_STATE_CHANGE " .. tostring(lastCoverType) .. " > " ..tostring(currentCoverType));
        commonlib_setGameParam(HLDDZ_KEY_GAME_RUNNING_COVER_STATUS, currentCoverType);
    end
end

--- 遮挡类型检查
local function getGameCoverType(currentGameState, lastGameState)
    local currentCoverType = HLDDZ_GAME_COVER_NONE;
    if currentGameState == HLDDZ_GAMESTATE_UNKNOW and lastGameState == HLDDZ_GAMESTATE_RUNNING then
        currentCoverType = HLDDZ_GAME_COVER_UNKNOWN;
    end
    return currentCoverType;
end

local function resetStatus()
    isMatchRun = false
    commonlib_setGameParam(HLDDZ_KEY_GAME_RUNNING_COVER_STATUS, HLDDZ_GAME_COVER_NONE);
    commonlib_setGameParam(HLDDZ_KEY_SWITCH_FLOAT_BALL_SHOW_OR_HID, true)
    commonlib_setGameParam(HLDDZ_KEY_GAME_OVER_SUCCESS, nil)
end

local function isGameHall()
    if isMatchRun == true then
        return false
    end

    local result = false
    result = gNewGameEngine:matchCondition("cd_build_game_in_hall")
    if result then

        isStart = nil
    end
    return result
end

local function isGameStart()
    if isMatchRun == false then
        return false
    end
    local result = gNewGameEngine:matchCondition("cd_build_game_start")
    if result == false then
        result = matchCaffe2Classify("cd_build_game_start_right")
    end
    if result == false then
        result = matchCaffe2Classify("cd_build_game_start_left")
    end
    if result then
        isStart = true

    end
    return result
end

local function isRunning()
    if isStart ~= nil and isStart == true and isMatchRun == true then
        return true
    end
    return false
end

local function isGameOver()
    if isMatchRun == true then
        return false
    end
    local result = matchCaffe2Classify("cd_build_game_over_failed")
    if result == false then
        result = matchCaffe2Classify("cd_build_game_over_success")
        if result == true then
            commonlib_setGameParam(HLDDZ_KEY_GAME_OVER_SUCCESS, true)
        end
    else
        commonlib_setGameParam(HLDDZ_KEY_GAME_OVER_SUCCESS, false)
    end
    if result then
        isStart = nil
    end
    return result
end

local function distpatchGameStateEvent(lastGameState, currentState)
    printLog("distpatchGameStateEvent:start, state " .. tostring(lastGameState) .. " > " .. tostring(currentState))
    if lastGameState ~= currentState then
        if (currentState ~= HLDDZ_GAMESTATE_UNKNOW and lastGameState == HLDDZ_GAMESTATE_RUNNING) then
            printLog("distpatchGameStateEvent:HLDDZ_KEY_GAME_OVER")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_GAME_OVER])
        end

        if currentState == HLDDZ_GAMESTATE_RUNNING then
            printLog("distpatchGameStateEvent:HLDDZ_KEY_ENTER_GAME")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_ENTER_GAME])
        end

        if currentState == HLDDZ_GAMESTATE_IN_GAMESTART then
            printLog("distpatchGameStateEvent:HLDDZ_KEY_ENTER_START")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_ENTER_START])
        end
    end

    if currentState == HLDDZ_GAMESTATE_RUNNING or
            (lastGameState == HLDDZ_GAMESTATE_RUNNING and currentState == HLDDZ_GAMESTATE_UNKNOW) then
        printLog("distpatchGameStateEvent:HLDDZ_KEY_GAME_RUNNING")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_GAME_RUNNING])
    elseif currentState == HLDDZ_GAMESTATE_IN_GAMESTART then
        printLog("distpatchGameStateEvent:HLDDZ_KEY_IN_GAME_START")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_IN_GAME_START])
    elseif currentState == HLDDZ_GAMESTATE_OVER then
        printLog("distpatchGameStateEvent:HLDDZ_KEY_GAME_OVER_PAGE")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_GAME_OVER_PAGE]);
    elseif currentState == HLDDZ_GAMESTATE_IN_HALL then
        printLog("distpatchGameStateEvent:HLDDZ_GAMESTATE_IN_HALL")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[HLDDZ_KEY_IN_GAME_HALL]);
    end

    if lastGameState ~= currentState then
        printLog("distpatchGameStateEvent:HLDDZ_KEY_GAME_STATUS_CHANGED")
        commonlib_handleTableWithTwoParam(common_game_events_callbacks[HLDDZ_KEY_GAME_STATUS_CHANGED],
                lastGameState, currentState);
    end

    printLog("distpatchGameStateEvent:end")
end

local function getCurrentState(lastState)
    local currentState = HLDDZ_GAMESTATE_UNKNOW;
    if lastState == nil or lastState == HLDDZ_GAMESTATE_UNKNOW then
        if isGameHall() then
            printLog("未知-->游戏大厅");
            currentState = HLDDZ_GAMESTATE_IN_HALL;
        elseif isGameStart() then
            printLog("未知-->游戏开始");
            currentState = HLDDZ_GAMESTATE_IN_GAMESTART;
        elseif isRunning() then
            printLog("未知-->游戏进行中");
            currentState = HLDDZ_GAMESTATE_RUNNING;
        elseif isGameOver() then
            printLog("未知-->游戏结束");
            currentState = HLDDZ_GAMESTATE_OVER;
        end
    elseif lastState == HLDDZ_GAMESTATE_IN_GAMESTART then
        if isGameStart() then
            printLog("游戏开始-->游戏开始");
            currentState = HLDDZ_GAMESTATE_IN_GAMESTART;
        elseif isRunning() then
            printLog("游戏开始-->游戏运行");
            currentState = HLDDZ_GAMESTATE_RUNNING;
        elseif isGameOver() then
            printLog("游戏开始-->游戏结束");
            currentState = HLDDZ_GAMESTATE_OVER;
        elseif isGameHall() then
            printLog("游戏开始-->游戏大厅");
            currentState = HLDDZ_GAMESTATE_IN_HALL;
        end
    elseif lastState == HLDDZ_GAMESTATE_RUNNING then
        if isRunning() then
            printLog("游戏运行-->游戏运行");
            currentState = HLDDZ_GAMESTATE_RUNNING;
        elseif isGameOver() then
            currentState = HLDDZ_GAMESTATE_OVER;
            printLog("游戏运行-->游戏结束");
        elseif isGameStart() then
            printLog("游戏运行-->游戏开始");
            currentState = HLDDZ_GAMESTATE_IN_GAMESTART;
        elseif isGameHall() then
            printLog("游戏运行-->游戏大厅");
            currentState = HLDDZ_GAMESTATE_IN_HALL;
        end
    elseif lastState == HLDDZ_GAMESTATE_OVER then
        if isGameOver() then
            currentState = HLDDZ_GAMESTATE_OVER;
            printLog("游戏结束-->游戏结束");
        elseif isGameStart() then
            printLog("游戏结束-->游戏开始");
            currentState = HLDDZ_GAMESTATE_IN_GAMESTART;
        elseif isRunning() then
            printLog("游戏结束-->游戏运行");
            currentState = HLDDZ_GAMESTATE_RUNNING;
        elseif isGameHall() then
            printLog("游戏结束-->游戏大厅");
            currentState = HLDDZ_GAMESTATE_IN_HALL;
        end
    elseif lastState == HLDDZ_GAMESTATE_IN_HALL then
        if isGameHall() then
            printLog("游戏大厅-->游戏大厅");
            currentState = HLDDZ_GAMESTATE_IN_HALL;
        elseif isGameStart() then
            printLog("游戏大厅-->游戏开始");
            currentState = HLDDZ_GAMESTATE_IN_GAMESTART;
        elseif isRunning() then
            printLog("游戏大厅-->游戏运行");
            currentState = HLDDZ_GAMESTATE_RUNNING;
        elseif isGameOver() then
            currentState = HLDDZ_GAMESTATE_OVER;
            printLog("游戏大厅-->游戏结束");
        end
    end
    return currentState;
end

local function setCurrentGameState(currentState, lastState)
    if currentState ~= lastState and currentState ~= HLDDZ_GAMESTATE_UNKNOW then
        commonlib_setGameParam(HLDDZ_KEY_GAME_STATE, currentState);
        printLog("GAME_STATE_CHANGE " .. tostring(lastState) .. " > " ..tostring(currentState));
    end
end

local function onGameHall()
    printLog("onGameHall");
    resetStatus()
end

local function onEnterGame()
    printLog("onEnterGame");
    if isCanTracePerformance then
        performanceTrace:onJustEnterGame();
    end
    resetStatus()
end

local function onGameOver()
    printLog("onGameOver");
    if isCanTracePerformance then
        performanceTrace:onGameOver();
    end
    resetStatus()
end

local function onGameRunning()

end

local function onGameStateChange(lastGameState, currentState)
    if lastGameState == nil or currentState == nil then
        return
    end

    if lastGameState == HLDDZ_GAMESTATE_IN_HALL and currentState == HLDDZ_GAMESTATE_UNKNOW then
        if reportWaitStartReport == false then
            local result = gNewGameEngine:matchCondition("cd_build_game_wait_start")
            if result == true then
                report_gamemaster_plugin_hlddz_main(11, 0, 0, 0,
                        "", "",0,0)
                reportWaitStartReport = true
            end
        end
    end
end

local function registerEvents()
    commonlib_registerCallback(common_game_events_callbacks[HLDDZ_KEY_IN_GAME_HALL], onGameHall)
    commonlib_registerCallback(common_game_events_callbacks[HLDDZ_KEY_ENTER_GAME], onEnterGame)
    commonlib_registerCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_RUNNING], onGameRunning)
    commonlib_registerCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_OVER], onGameOver)
    commonlib_registerCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_STATUS_CHANGED], onGameStateChange)
end

local function unRegisterEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[HLDDZ_KEY_IN_GAME_HALL], onGameHall)
    commonlib_unregisterCallback(common_game_events_callbacks[HLDDZ_KEY_ENTER_GAME], onEnterGame)
    commonlib_unregisterCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_RUNNING], onGameRunning)
    commonlib_unregisterCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_OVER], onGameOver)
    commonlib_unregisterCallback(common_game_events_callbacks[HLDDZ_KEY_GAME_STATUS_CHANGED], onGameStateChange)
end

local function work()
    -- 游戏主状态
    if getHlddzFuncCloudSwitch() == false then
        printLog("hlddz enable is false")
        return
    end


    gNewGameEngine:clearCache()
    isMatchRun = gNewGameEngine:matchCondition("cd_build_game_run")
    local lastGameState = commonlib_getGameParam(HLDDZ_KEY_GAME_STATE)
    local currentGameState = getCurrentState(lastGameState);
    setCurrentGameState(currentGameState, lastGameState)

    -- 遮挡类型
    local lastCoverType = commonlib_getGameParam(HLDDZ_KEY_GAME_RUNNING_COVER_STATUS);
    local currentCoverType = getGameCoverType(currentGameState, lastGameState);
    setCurrentGameCoverType(currentCoverType, lastCoverType)
    distpatchGameStateEvent(lastGameState, currentGameState)
end

local function callLifeCircleInit()
    printLog("callLifeCircleInit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleInit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, INIT_INDEX)
end

local function callLifeCircleUninit()
    printLog("callLifeCircleUninit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleUninit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, UNINIT_INDEX)
end

local function initPerformanceTrace()
    performanceTrace = PerformanceTrace()
    isCanTracePerformance = performanceTrace:canTracePerformance()
end

local function initVariable()
    gNewGameEngine = CNewGameEngine:getInstance();
    printLog("init variable," .. tostring(gNewGameEngine))

    commonlib_setGameParam(HLDDZ_KEY_GAME_STATE, HLDDZ_GAMESTATE_UNKNOW);
    commonlib_setGameParam(HLDDZ_KEY_GAME_RUNNING_COVER_STATUS, HLDDZ_GAME_COVER_NONE);
end

local function init()
    printLog("LuaFlow: HLDDZ init start")
    initVariable();
    resetStatus()
    commonlib_lifeCircleSort()
    callLifeCircleInit()
    registerEvents()
    commonlib_putSettingViewsEventListener()
    initPerformanceTrace()
    printLog("LuaFlow: HLDDZ init end")
    init_feedback()
end

local function uninit()
    printLog("LuaFlow: HLDDZ uninit start")
    callLifeCircleUninit()
    unRegisterEvents()
    printLog("LuaFlow: HLDDZ uninit end")
end

table.insert(core_scripts, { init, uninit, work })

