--- 游戏状态脚本
--- 识别背包
local BACKPACK_RECOGNIZE_INFO = { path = "img/commonstate/refer_backpack.png", rect = { flag = Anchor().BOTTOMLEFT, left = 34, top = -102, width = 61, height = 65 }, threshold = 98 };
--- 识别左侧枪支按钮
local GUN_LEFT_RECOGNIZE_INFO = { path = "img/commonstate/refer_gun_left.png", rect = { flag = Anchor().BOTTOMLEFT, left = 39, top = -452, width = 78, height = 78 }, threshold = 96 };
--- 识别右侧枪支按钮
local GUN_RIGHT_RECOGNIZE_INFO = { path = "img/commonstate/refer_gun_right.png", rect = { flag = Anchor().BOTTOMRIGHT, left = -336, top = -338, width = 109, height = 109 }, threshold = 97 };
--- 识别左侧拳头按钮
local FIST_LEFT_RECOGNIZE_INFO = { path = "img/commonstate/refer_fist_left.png", rect = { flag = Anchor().BOTTOMLEFT, left = 41, top = -448, width = 78, height = 65 }, threshold = 96 };
--- 识别右侧拳头按钮
local FIST_RIGHT_RECOGNIZE_INFO = { path = "img/commonstate/refer_fist_right.png", rect = { flag = Anchor().BOTTOMRIGHT, left = -340, top = -334, width = 111, height = 91 }, threshold = 97 };

local FLOAT_BALL_POSITION = { anchor = Anchor().TOPLEFT, x = 71, y = 455 }

local Tag = "common_state"
local function printLog(msg)
    commonlib_print(Tag, msg);
end

KEY_BACKPACK = "backpack"
KEY_RIGHT_GUN = "rightGun"
KEY_RIGHT_FIST = "rightFist"
KEY_LEFT_GUN = "leftGun"
KEY_LEFT_FIST = "leftFist"

CLDTS_APP_NAME = 3

local floatBall = TopSmallView();
local hasReportGameStartData = false;

local hasIconTable = {
    [KEY_BACKPACK] = nil,
    [KEY_RIGHT_GUN] = nil,
    [KEY_RIGHT_FIST] = nil,
    [KEY_LEFT_GUN] = nil,
    [KEY_LEFT_FIST] = nil
}

-- 用于回调到各个业务脚本的回调函数表
common_game_events_callbacks = {
    [KEY_BACKPACK] = {},
    [KEY_RIGHT_GUN] = {},
    [KEY_RIGHT_FIST] = {},
    [KEY_LEFT_GUN] = {},
    [KEY_LEFT_FIST] = {}
}

local INIT_INDEX = 1
local UNINIT_INDEX = 2

local function callLifeCircleInit()
    printLog("callLifeCircleInit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleInit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, INIT_INDEX)
end

local DEFAULT_ALGORITHM = 2
local function doImageRecognize(info)
    return commonlib_doImageRecognizeByType(info, DEFAULT_ALGORITHM)
end

local function callLifeCircleUninit()
    printLog("callLifeCircleUninit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleUninit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, UNINIT_INDEX)
end

local function setSettingViewIcon()
    commonlib_setFloatBallIcon("common://image/icon_floatball.png")
    commonlib_setSettingViewIcon("common://image/icon_settingview.png")
    commonlib_setSettingViewIconSize(97.67, 65)
    commonlib_setSettingViewIconMargin(12, 0, 0, 0)
    commonlib_setSettingViewIconExtrudeHeight(21)
end

local function init()
    printLog("init")

    commonlib_recognizeAddImage(BACKPACK_RECOGNIZE_INFO.path);

    commonlib_recognizeAddImage(FIST_LEFT_RECOGNIZE_INFO.path);
    commonlib_recognizeAddImage(FIST_RIGHT_RECOGNIZE_INFO.path);

    commonlib_recognizeAddImage(GUN_LEFT_RECOGNIZE_INFO.path);
    commonlib_recognizeAddImage(GUN_RIGHT_RECOGNIZE_INFO.path);

    callLifeCircleInit();

    -- 开始设置悬浮球的外观
    setSettingViewIcon()

    floatBall:moveCenterTo(FLOAT_BALL_POSITION.anchor, FLOAT_BALL_POSITION.x, FLOAT_BALL_POSITION.y)
    floatBall:hide()

    commonlib_putSettingViewsEventListener()
end

local function uninit()
    printLog("uninit")
    callLifeCircleUninit();
    hasReportGameStartData = false;
end

local function handleEventCallbacks(iconKey, hasIcon)
    printLog("handleEventCallbacks : " .. tostring(iconKey) .. " : " .. tostring(hasIcon))
    if hasIconTable[iconKey] == nil or hasIconTable[iconKey] ~= hasIcon then
        hasIconTable[iconKey] = hasIcon;
        commonlib_setGameParam(iconKey, hasIcon)
        commonlib_handleTableWithBoolParam(common_game_events_callbacks[iconKey], hasIconTable[iconKey])
    end
end

local function reportGameStartData()
    if hasReportGameStartData then
        commonlib_report(GAMEMASTER_FUNCTION_DISABLE_REASON_TABLE,
            {
                [GAMEMASTER_FUNCTION_DISABLE_REASON_STEP] = STEP_GAME_INNER_START,
                [GAMEMASTER_FUNCTION_DISABLE_REASON_GAMENAME] = CLDTS_APP_NAME,
                [GAMEMASTER_FUNCTION_DISABLE_REASON] = REASON_SUCCEED
            });
        hasReportGameStartData = true;
    end
end

local function work()
    printLog("work")

    local hasRightGunIcon = false;
    local hasRightFistIcon = false;
    local hasLeftGunIcon = false;
    local hasLeftFistIcon = false;

    hasRightGunIcon = doImageRecognize(GUN_RIGHT_RECOGNIZE_INFO);
    if not hasRightGunIcon then
        hasRightFistIcon = doImageRecognize(FIST_RIGHT_RECOGNIZE_INFO);
    end

    if hasRightGunIcon then
        hasLeftGunIcon = doImageRecognize(GUN_LEFT_RECOGNIZE_INFO);
    elseif hasRightFistIcon then
        hasLeftFistIcon = doImageRecognize(FIST_LEFT_RECOGNIZE_INFO);
    else
        hasLeftGunIcon = doImageRecognize(GUN_LEFT_RECOGNIZE_INFO);
        if not hasLeftGunIcon then
            hasLeftFistIcon = doImageRecognize(FIST_LEFT_RECOGNIZE_INFO);
        end
    end

    local hasBackpackIcon = hasLeftFistIcon or hasLeftGunIcon or hasRightFistIcon or hasRightGunIcon
    commonlib_runOnUIThread(function()
        --if hasIconTable[KEY_BACKPACK] ~= hasBackpackIcon then
        printLog("开始显示悬浮球：" .. tostring(hasBackpackIcon));
        if hasBackpackIcon then
            reportGameStartData();
            if not floatBall:isShowing() then
                floatBall:show();
            end
        else
            if floatBall:isShowing() then
                floatBall:hide();
            end
        end
        --end
    end);

    printLog("开始检测是否背包按钮：" .. tostring(hasBackpackIcon))

    handleEventCallbacks(KEY_BACKPACK, hasBackpackIcon);

    if not hasBackpackIcon then
        printLog("没有背包按钮，不做任何操作")
        for index, v in pairs(hasIconTable) do
            hasIconTable[index] = false;
            commonlib_setGameParam(index, false)
        end
        return
    end

    handleEventCallbacks(KEY_RIGHT_GUN, hasRightGunIcon);
    handleEventCallbacks(KEY_RIGHT_FIST, hasRightFistIcon);
    handleEventCallbacks(KEY_LEFT_GUN, hasLeftGunIcon);
    handleEventCallbacks(KEY_LEFT_FIST, hasLeftFistIcon);
end

table.insert(core_scripts, { init, uninit, work })