
local function printLog(msg)
    commonlib_print("common_state", msg)
end

local INIT_INDEX = 1
local UNINIT_INDEX = 2

--- ******游戏运行状态相关******----
PUBGM_KEY_GAME_STATE = "gameState";
PUBGM_GAMESTATE_UNKNOW = -2; --未知
PUBGM_GAMESTATE_IN_GAMEHOME = -1;--游戏大厅
PUBGM_GAMESTATE_LOADING = 0;--游戏加载状态，识别模式
PUBGM_GAMESTATE_RUNNING = 1;--游戏进行中
PUBGM_GAMESTATE_OVER = 2;--结束游戏

PUBGM_KEY_IN_GAME_HOME = "PubgmInGameHome"--HOME页
PUBGM_KEY_ENTER_HOME = "PubgmKeyEnterHome" --进入游戏HOME页
PUBGM_KEY_ENTER_GAME = "PubgmEnterGame"--进入游戏
PUBGM_KEY_GAME_RUNNING = "PubgmGameRunning"--游戏运行中
PUBGM_KEY_GAME_OVER = "PubgmGameOver"--游戏结束
PUBGM_KEY_GAME_OVER_PAGE = "PubgmGameOverPage"--在游戏结页面
PUBGM_KEY_GAME_STATUS_CHANGED = "PubgmGameStatusChanged"--游戏状态变更
KEY_GAME_WORK = "gameWork"

PUBGM_KEY_GAME_RUNNING_COVER_STATUS = "PubgmRuningCoverStatus"
--- 遮挡类型
PUBGM_GAME_COVER_NONE = 0;    -- 无遮挡
PUBGM_GAME_COVER_UNKNOWN = 1; -- 未知遮挡

PUBGM_KEY_GAME_MODE = "PubgmGameMode"
PUBGM_GAME_MODEL_UNKNOW = -1
PUBGM_GAME_MODEL_SMOOTH = 0;
PUBGM_GAME_MODEL_GOOD_QUALITY = 1;
PUBGM_KEY_ON_GAME_MODE_CONFIRMED = "PubgmOnGameModeConfirmed"

-- 用于回调到各个业务脚本的回调函数表
common_game_events_callbacks = {
    [PUBGM_KEY_ENTER_GAME] = {},
    [PUBGM_KEY_GAME_RUNNING] = {},
    [PUBGM_KEY_GAME_OVER] = {},
    [PUBGM_KEY_IN_GAME_HOME] = {},
    [PUBGM_KEY_ENTER_HOME] = {},
    [PUBGM_KEY_GAME_STATUS_CHANGED]={},
    [PUBGM_KEY_GAME_OVER_PAGE] = {},
    [PUBGM_KEY_ON_GAME_MODE_CONFIRMED] = {},
    [KEY_GAME_WORK] = {},
}



local gNewGameEngine;
local gNeedReInitGameMode = true;
local gCurrentCheckRunningStateKeyName

local MAX_UPLOAD_BITMAP_COUNT = 1
local uploadBitmapCount = 0
local enterHomeTime = 0

local function setCurrentGameCoverType(currentCoverType, lastCoverType)
    if lastCoverType ~= currentCoverType then
        printLog("COVER_STATE_CHANGE " .. tostring(lastCoverType) .. " > " ..tostring(currentCoverType));
        commonlib_setGameParam(PUBGM_KEY_GAME_RUNNING_COVER_STATUS, currentCoverType);
    end
end

--- 遮挡类型检查
local function getGameCoverType(currentGameState, lastGameState)
    local currentCoverType = PUBGM_GAME_COVER_NONE;
    if currentGameState == PUBGM_GAMESTATE_UNKNOW and lastGameState == PUBGM_GAMESTATE_RUNNING then
        currentCoverType = PUBGM_GAME_COVER_UNKNOWN;
    end
    return currentCoverType;
end

local function resetStatus()
    commonlib_setGameParam(PUBGM_KEY_GAME_RUNNING_COVER_STATUS, PUBGM_GAME_COVER_NONE);
    gNeedReInitGameMode = true
end

local function isInHallHome()
    local result = gNewGameEngine:matchCondition("cd_build_group_in_home")
    --printLog("isInHallHome: " .. tostring(result))
    return result
end


local function initGameMode()
    local gameMode
    local isBigMap = false;

    if gNewGameEngine:matchCondition("cd_build_setting") then
        gameMode = PUBGM_GAME_MODEL_GOOD_QUALITY;
    elseif gNewGameEngine:matchCondition("cd_build_setting") then
        gameMode = PUBGM_GAME_MODEL_SMOOTH;
    end

    if gameMode == nil then
        return false;
    end

    gNewGameEngine:clearCache({CLEAR_FLAG_FORCE});
    gNewGameEngine:matchCondition("cd_build_setting");
    gNewGameEngine:matchCondition("cd_build_setting");

    if gameMode == PUBGM_GAME_MODEL_GOOD_QUALITY then
        gCurrentCheckRunningStateKeyName = "cd_build_setting";

    else
        gCurrentCheckRunningStateKeyName = "cd_build_setting";
    end

    commonlib_setGameParam(PUBGM_KEY_GAME_MODE, gameMode);
    commonlib_handleTableWithOneParam(common_game_events_callbacks[PUBGM_KEY_ON_GAME_MODE_CONFIRMED], gameMode);

    printLog("setGameMode:initGameMode:gameMode="..tostring(gameMode)..",isBigMap="..tostring(isBigMap));

    return true;
end

local function isRunning()
    local result = false
    if gNeedReInitGameMode then
        result = initGameMode();
        if result == true then
            gNeedReInitGameMode = false;
        end
    else
        result = gNewGameEngine:matchCondition(gCurrentCheckRunningStateKeyName)
    end
    --printLog("isRunning: " .. tostring(result))
    return result
end

local function isGameOver()
    local result = gNewGameEngine:matchCondition("cd_build_game_over")
    --printLog("isGameOver: " .. tostring(result))
    return result
end

local function distpatchGameStateEvent(lastGameState, currentState)
    printLog("distpatchGameStateEvent:start, state " .. tostring(lastGameState) .. " > " .. tostring(currentState))

    if lastGameState ~= currentState then
        if (lastGameState == PUBGM_GAMESTATE_RUNNING and currentState == PUBGM_GAMESTATE_OVER) or
                (currentState ~= PUBGM_GAMESTATE_UNKNOW and lastGameState == PUBGM_GAMESTATE_RUNNING) then
            printLog("distpatchGameStateEvent:PUBGM_KEY_GAME_OVER")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_GAME_OVER])
        end

        if currentState == PUBGM_GAMESTATE_RUNNING then
            printLog("distpatchGameStateEvent:PUBGM_KEY_ENTER_GAME")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_ENTER_GAME])
        end

        if currentState == PUBGM_GAMESTATE_IN_GAMEHOME then
            printLog("distpatchGameStateEvent:PUBGM_KEY_ENTER_HOME")
            commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_ENTER_HOME])
        end
    end

    if currentState == PUBGM_GAMESTATE_RUNNING or
            (lastGameState == PUBGM_GAMESTATE_RUNNING and currentState == PUBGM_GAMESTATE_UNKNOW) then
        printLog("distpatchGameStateEvent:PUBGM_KEY_GAME_RUNNING")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_GAME_RUNNING])
    elseif currentState == PUBGM_GAMESTATE_IN_GAMEHOME then
        printLog("distpatchGameStateEvent:PUBGM_KEY_IN_GAME_HOME")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_IN_GAME_HOME])
    elseif currentState == PUBGM_GAMESTATE_OVER then
        printLog("distpatchGameStateEvent:PUBGM_KEY_GAME_OVER_PAGE")
        commonlib_handleTableWithoutParam(common_game_events_callbacks[PUBGM_KEY_GAME_OVER_PAGE]);
    end

    if lastGameState ~= currentState then
        printLog("distpatchGameStateEvent:PUBGM_KEY_GAME_STATUS_CHANGED")
        commonlib_handleTableWithTwoParam(common_game_events_callbacks[PUBGM_KEY_GAME_STATUS_CHANGED],
                lastGameState, currentState);
    end

    printLog("distpatchGameStateEvent:end")
end

local function getCurrentState(lastState)
    local currentState = PUBGM_GAMESTATE_UNKNOW;
    if lastState == nil or lastState == PUBGM_GAMESTATE_UNKNOW then
        if isInHallHome() then
            printLog("未知-->游戏大厅");
            currentState = PUBGM_GAMESTATE_IN_GAMEHOME;
        elseif isRunning() then
            printLog("未知-->游戏进行中");
            currentState = PUBGM_GAMESTATE_RUNNING;
        elseif isGameOver() then
            printLog("未知-->游戏结束");
            currentState = PUBGM_GAMESTATE_OVER;
        end
    elseif lastState == PUBGM_GAMESTATE_IN_GAMEHOME then
        if isInHallHome() then
            printLog("游戏大厅-->游戏大厅");
            currentState = PUBGM_GAMESTATE_IN_GAMEHOME;
        elseif isRunning() then
            printLog("游戏大厅-->游戏运行");
            currentState = PUBGM_GAMESTATE_RUNNING;
        elseif isGameOver() then
            printLog("游戏大厅-->游戏结束");
            currentState = PUBGM_GAMESTATE_OVER;
        end
    elseif lastState == PUBGM_GAMESTATE_RUNNING then
        if isRunning() then
            printLog("游戏运行-->游戏运行");
            currentState = PUBGM_GAMESTATE_RUNNING;
        elseif isGameOver() then
            currentState = PUBGM_GAMESTATE_OVER;
            printLog("游戏运行-->游戏结束");
        elseif isInHallHome() then
            printLog("游戏运行-->游戏大厅");
            currentState = PUBGM_GAMESTATE_IN_GAMEHOME;
        end
    elseif lastState == PUBGM_GAMESTATE_OVER then
        if isGameOver() then
            currentState = PUBGM_GAMESTATE_OVER;
            printLog("游戏结束-->游戏结束");
        elseif isInHallHome() then
            printLog("游戏结束-->游戏大厅");
            currentState = PUBGM_GAMESTATE_IN_GAMEHOME;
        elseif isRunning() then
            printLog("游戏结束-->游戏运行");
            currentState = PUBGM_GAMESTATE_RUNNING;
        end
    end
    return currentState;
end

local function setCurrentGameState(currentState, lastState)
    if currentState ~= lastState and currentState ~= PUBGM_GAMESTATE_UNKNOW then
        commonlib_setGameParam(PUBGM_KEY_GAME_STATE, currentState);
        printLog("GAME_STATE_CHANGE " .. tostring(lastState) .. " > " ..tostring(currentState));
    end
end

local function onEnterGame()
    printLog("onEnterGame");
    enterHomeTime = commonlib_getCurrentTime()
    common_skin_work()
    resetStatus()
end

local function onGameOver()
    printLog("onGameOver");
    common_skin_work()
    resetStatus()
end

local function onGameRunning()
    common_skin_work()
end

local function registerEvents()
    commonlib_registerCallback(common_game_events_callbacks[PUBGM_KEY_ENTER_GAME], onEnterGame)
    commonlib_registerCallback(common_game_events_callbacks[PUBGM_KEY_GAME_RUNNING], onGameRunning)
    commonlib_registerCallback(common_game_events_callbacks[PUBGM_KEY_GAME_OVER], onGameOver)
end

local function unRegisterEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[PUBGM_KEY_ENTER_GAME], onEnterGame)
    commonlib_unregisterCallback(common_game_events_callbacks[PUBGM_KEY_GAME_RUNNING], onGameRunning)
    commonlib_unregisterCallback(common_game_events_callbacks[PUBGM_KEY_GAME_OVER], onGameOver)
end

local function work()
    --printLog("work:start")
    -- 游戏主状态
    gNewGameEngine:clearCache()

    local lastGameState = commonlib_getGameParam(PUBGM_KEY_GAME_STATE)
    local currentGameState = getCurrentState(lastGameState);
    setCurrentGameState(currentGameState, lastGameState)

    if lastGameState ~= currentGameState then
	    gNeedReInitGameMode = true
        gNewGameEngine:clearCache({CLEAR_FLAG_FORCE});
    end

    -- 遮挡类型
    local lastCoverType = commonlib_getGameParam(PUBGM_KEY_GAME_RUNNING_COVER_STATUS);
    local currentCoverType = getGameCoverType(currentGameState, lastGameState);
    setCurrentGameCoverType(currentCoverType, lastCoverType);

    distpatchGameStateEvent(lastGameState, currentGameState)
    commonlib_handleTableWithoutParam(common_game_events_callbacks[KEY_GAME_WORK])
end

local function callLifeCircleInit()
    printLog("callLifeCircleInit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleInit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, INIT_INDEX)
end

local function callLifeCircleUninit()
    printLog("callLifeCircleUninit begin")
    if commonlib_isTableEmpty(common_life_circle_callbacks) then
        printLog("callLifeCircleUninit common_life_circle_callbacks is nil or empty")
        return
    end
    commonlib_handleNestingTable(common_life_circle_callbacks, UNINIT_INDEX)
end

local function initVariable()
    gNewGameEngine = CNewGameEngine:getInstance();
    commonlib_setGameParam(PUBGM_KEY_GAME_STATE, PUBGM_GAMESTATE_UNKNOW);
    commonlib_setGameParam(PUBGM_KEY_GAME_RUNNING_COVER_STATUS, PUBGM_GAME_COVER_NONE);
    report_gamemaster_game_pubgm_state(100, 1)
end

local function init()
    printLog("LuaFlow: PUBGM init start")
    initVariable();
    resetStatus()
    commonlib_lifeCircleSort()
    callLifeCircleInit()
    registerEvents()
    commonlib_putSettingViewsEventListener()
    common_skin_init()
    printLog("LuaFlow: PUBGM init end")
	init_feedback()    
end

local function uninit()
    printLog("LuaFlow: PUBGM uninit start")
    callLifeCircleUninit()
    unRegisterEvents()
    common_skin_uninit()
    printLog("LuaFlow: PUBGM uninit end")
end

table.insert(core_scripts, { init, uninit, work })

