--
-- Created by IntelliJ IDEA.
-- User: zys
-- Date: 2018/5/3
-- Time: 11:44
-- To change this template use File | Settings | File Templates.
--
--- gamemaster_pubgm:411 plugin:byte state:byte erro:byte game_time:int64 package:string

local ASSIST_SHOW_REPORT_TAG = "assist_show_report"

ASSIST_SHOW_TYPE_ENTER_GAME  = 1  --- ok
ASSIST_SHOW_TYPE_GUN_SIGHT   = 2  --- ok
ASSIST_SHOW_TYPE_GUN_SKIN    = 3  --- ok
ASSIST_SHOW_TYPE_FLOW_VIEW   = 4  --- ok
ASSIST_SHOW_TYPE_RADAR_RING  = 5  --- ok
ASSIST_SHOW_TYPE_MAP_RES     = 6  --- ok
ASSIST_SHOW_TYPE_MAP_ROUTE   = 7  --- ok
ASSIST_SHOW_TYPE_STATIC_SKIN = 8  --- ok
ASSIST_SHOW_TYPE_BOTTOM_GUN_SKIN  = 9  --- ok
ASSIST_SHOW_TYPE_BOTTOM_DRUG_SKIN = 10  --- ok
ASSIST_SHOW_TYPE_RADAR_CICLE      = 11  --- ok
ASSIST_SHOW_TYPE_RADAR_TOAST      = 12  --- ok
ASSIST_SHOW_TYPE_ARSENAL          = 13  --- ok

local ASSIST_SHOW_STATE_SUCCEED   = 1
local ASSIST_SHOW_STATE_FAILED    = 2
local ASSIST_SHOW_STATE_NOTRIGGER = 3

ASSIST_SHOW_ERROR_UNKNOWN        = 0
ASSIST_SHOW_ERROR_FUNCTION_CLOSE = 1
ASSIST_SHOW_ERROR_RECOGNIZE      = 2


local ASSIST_SHOW_REPORT_TABLE = "gamemaster_pubgm"
local ASSIST_SHOW_REPORT_KEY_PLUGIN = "plugin"
local ASSIST_SHOW_REPORT_KEY_STATE  = "state"
local ASSIST_SHOW_REPORT_KEY_ERROR  = "erro"
local ASSIST_SHOW_REPORT_KEY_GAME_T = "game_time"
local ASSIST_SHOW_REPORT_KEY_PACKAGE = "package"


local function printLog(msg)
    commonlib_print(ASSIST_SHOW_REPORT_TAG, msg)
end

----------------------------------
--- cache
AssistShowReportCache = {
}

function AssistShowReportCache:new()
    local object = {
        mbReport   = false,
        mnState    = ASSIST_SHOW_STATE_NOTRIGGER,
        mnError    = ASSIST_SHOW_ERROR_UNKNOWN,
    }

    setmetatable(object, self);
    self.__index = self;

    return object;
end

function AssistShowReportCache:reset()
    self.mbReport   = false
    self.mnState    = ASSIST_SHOW_STATE_NOTRIGGER
    self.mnError    = ASSIST_SHOW_ERROR_UNKNOWN
end

function AssistShowReportCache:setReport(bReport)
    self.mbReport = bReport
end
function AssistShowReportCache:isReport()
    return self.mbReport
end

function AssistShowReportCache:setState(nState)
    self.mnState = nState
end
function AssistShowReportCache:getState()
    return self.mnState
end

function AssistShowReportCache:setError(nError)
    self.mnError = nError
end
function AssistShowReportCache:getError()
    return self.mnError
end
---------------------------------

local CACHE_TABLE = {
    [ASSIST_SHOW_TYPE_ENTER_GAME]  = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_GUN_SIGHT]   = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_GUN_SKIN]    = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_FLOW_VIEW]   = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_RADAR_RING]  = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_MAP_RES]     = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_MAP_ROUTE]   = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_STATIC_SKIN] = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_BOTTOM_GUN_SKIN]  = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_BOTTOM_DRUG_SKIN] = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_RADAR_CICLE]      = AssistShowReportCache:new(),
    [ASSIST_SHOW_TYPE_RADAR_TOAST]      = AssistShowReportCache:new(),
}
local glEnterGameSysTime = -1
local glGameOverSysTime  = -1

local function internalReportAssistShow(type, cache)
    if nil ~= type and nil ~= cache and not cache:isReport() then
        cache:setReport(true)

        local lTimeS  = 0
        local lTimeMS = 0
        if glEnterGameSysTime > 0 and glGameOverSysTime > 0 then
            lTimeMS = glGameOverSysTime - glEnterGameSysTime
            local intPart,decimalPart = math.modf(lTimeMS / 1000)
            lTimeS = intPart
        end

        commonlib_report(ASSIST_SHOW_REPORT_TABLE,
            {
                [ASSIST_SHOW_REPORT_KEY_PLUGIN] = type,
                [ASSIST_SHOW_REPORT_KEY_STATE]  = cache:getState(),
                [ASSIST_SHOW_REPORT_KEY_ERROR]  = cache:getError(),
                [ASSIST_SHOW_REPORT_KEY_GAME_T] = lTimeS,
                [ASSIST_SHOW_REPORT_KEY_PACKAGE] = commonlib_getGamePackageName(),
            });

        printLog("report, type:" .. tostring(type) .. ", state:" .. tostring(cache:getState())
                .. ", error:" .. tostring(cache:getError()) .. ", report_state:" .. tostring(cache:isReport())
                .. ", game_time:" .. tostring(lTimeS) .. ", game_time_ms:" .. tostring(lTimeMS)
                .. ", package:" .. tostring( commonlib_getGamePackageName()))
    end
end

local function isEnterGame(type)
    if type == ASSIST_SHOW_TYPE_ENTER_GAME then
        return true
    end

    local cache = CACHE_TABLE[ASSIST_SHOW_TYPE_ENTER_GAME]
    return nil ~= cache and ASSIST_SHOW_STATE_SUCCEED == cache:getState()
end

function notifyEnterGame()
    glEnterGameSysTime = commonlib_getCurrentTime()
end
function notifyGameOver()
    glGameOverSysTime = commonlib_getCurrentTime()
end

function notifyAssistShowSucceed(type)
    if not isEnterGame(type) then
        return
    end

    local cache = CACHE_TABLE[type]
    if nil == cache or cache:isReport() or ASSIST_SHOW_STATE_SUCCEED == cache:getState() then
        return
    end

    cache:setState(ASSIST_SHOW_STATE_SUCCEED)
    cache:setError(ASSIST_SHOW_ERROR_UNKNOWN)
    printLog("[succeed] notifyAssistShowSucceed, type:" .. tostring(type)
            .. ", state:" .. tostring(cache:getState()) .. ", error:" .. tostring(cache:getError()))
end

function notifyAssistSwitchToOpen(type)
    if not isEnterGame(type) then
        return
    end

    local cache = CACHE_TABLE[type]
    if nil == cache or cache:isReport() or ASSIST_SHOW_STATE_SUCCEED == cache:getState() then
        return
    end

    cache:setState(ASSIST_SHOW_STATE_NOTRIGGER)
    cache:setError(ASSIST_SHOW_ERROR_UNKNOWN)
    printLog("[succeed] notifyAssistSwitchToOpen, type:" .. tostring(type)
            .. ", state:" .. tostring(cache:getState()) .. ", error:" .. tostring(cache:getError()))
end

function notifyAssistShowError(type, bSwitchOpen)
    if not isEnterGame(type) then
        return
    end

    local cache = CACHE_TABLE[type]
    if nil == cache or cache:isReport() or ASSIST_SHOW_STATE_SUCCEED == cache:getState() then
        return
    end

    local error = ASSIST_SHOW_ERROR_RECOGNIZE
    if not bSwitchOpen then
        error = ASSIST_SHOW_ERROR_FUNCTION_CLOSE
    end

    if error == cache:getError() and ASSIST_SHOW_STATE_FAILED == cache:getState() then
        return
    end

    cache:setState(ASSIST_SHOW_STATE_FAILED)
    cache:setError(error)
    printLog("[succeed] notifyAssistShowError, type:" .. tostring(type) .. ", switch_open:" .. tostring(bSwitchOpen)
            .. ", state:" .. tostring(cache:getState()) .. ", error:" .. tostring(cache:getError()))
end

function reportAllAssistShow()
    if not isEnterGame(nil) then
        printLog("reportAllAssistShow, not enter game")
        return
    end

    printLog("========= reportAllAssistShow start ==========")
    for type, cache in pairs(CACHE_TABLE) do
        if nil ~= type and nil ~= cache and not cache:isReport() then
            internalReportAssistShow(type, cache)
        end
    end
    printLog("=========  reportAllAssistShow end  ==========")
end

function reportAssistShow(type)
    if not isEnterGame(type) then
        printLog("reportAssistShow, not enter game, type:" .. tostring(type))
        return
    end

    internalReportAssistShow(type, CACHE_TABLE[type])
end

function resetAssistShowReportCache()
    printLog("========= resetAssistShowReportCache start ==========")
    for type, cache in pairs(CACHE_TABLE) do
        if nil ~= type and nil ~= cache then
            cache:setReport(false)
            cache:setState(ASSIST_SHOW_STATE_NOTRIGGER)
            cache:setError(ASSIST_SHOW_ERROR_UNKNOWN)

            printLog("reset, report:" .. tostring(cache:isReport()) .. ", state:" .. tostring(cache:getState())
                    .. ", error:" .. tostring(cache:getError()))
        end
    end

    glEnterGameSysTime = -1
    glGameOverSysTime  = -1
    printLog("=========  resetAssistShowReportCache end  ==========")
end



