--- 敌人提醒
---date:2018-04-11
---author: hexujun@cmcm.com

local GAMEMASTER_PUBGM_RADAR_TAG_TABLE = "gamemaster_pubgm_radar_tag"

local BATTLE_TAG = "battle_notice"
local floatWindow;
local battleNoticeData;
local gRecognizeAdapterManager;

local TAB_NAME = "吃鸡雷达"
local OBJECT_ANGLE_LIMIT = 2;--物体角度变化:> OBJECT_ANGLE_LIMIT 才认为不是同一个物体
local USER_ANGLE_LIMIT = 2;--用户角度变化> USER_ANGLE_LIMIT时，通知外部

local SHOW_RADAR = 1;
local SHOW_REMIND_TIPS = 2;
local SHOW_ALL = 0;
local NOT_SHOW_ALL = -1;

local gRadarSwitch = true
local gRadarCheckBox
local gRadarDescTextView
local RADAR_CHECKBOX_TEXT = "开启声音雷达"
local RADAR_DESC_TEXT = "声音雷达会将小地图上的声音标记映射到你的身边，并提供了角度支持"
local gRadarCheckBoxRedDot

local currentRingViewSwitch = SHOW_ALL;

local gRemindSwitch = true
local gRemindCheckBox
local gRemindDescTextView
local REMIND_CHECKBOX_TEXT = "开启警觉提醒"
local REMIND_DESC_TEXT = "当你身边出现声音的时候，会有一个提示横幅给你提示"

local OBJECT_END_TIME_LIMIT  = 3000 - 200  -- 判断物体是自然消失还是位置转移的时间 >= 2800ms则认为是自然消失
local GUN_DELAY_TIME         = 2000        -- 枪声的展现延迟时间
local MUFFLER_DELAY_TIME     = 2000        -- 消音器的展现延迟时间
local FOOT_DELAY_TIME        = 0           -- 脚步的展现延迟时间
local CARRIER_DELAY_TIME     = 0           -- 载具的展现延迟时间
local GLASS_DELAY_TIME       = 0           -- 玻璃的展现延迟时间

local AVE_S_MAX              = 70  --- 饱和度
local AVE_V_MAX              = 60  --- 亮度
local DISTANT_OBJECT_ALPHA   = 0.7

local AFTER_AIRPORT_END_SYS_TIME_DETECT_INFO = 30 * 1000  --- 离开机场（机场倒计时为0）多久后识别信息

local hasInCardOrShip = false;
local mLastRingInHorizontalCenter = false;
local hasDetectMap = false;
local mapType = 0;
local gNewGameEngine;

local AFTER_AIRPORT_END_SYS_TIME_SHOW_RING_VIEW = 40 * 1000

-- 用于控制展现角度及物体信息，方便调试，默认关掉
local TIP_SHOW_ENABLE = false

local MAX_UPLOAD_BITMAP_COUNT = 1
local uploadBitmapCount = 0
local enterHomeTime = 0

local function printLog(msg)
    commonlib_print(BATTLE_TAG, msg)
end

local function IsInTable(tableValue, value)
    if tableValue == nil then
        return false;
    end

    for k,v in ipairs(tableValue) do
        if v == value then
            return true;
        end
    end

    return false;
end

local function getMapType()
    if hasDetectMap ~= true then
        local map = commonlib_getGameParam(KEY_NAME_MAP_TYPE);
        if map == KEY_NAME_MAP_ISLAND then
            mapType = 1;
        elseif map == KEY_NAME_MAP_DESERT then
            mapType = 2;
        elseif map == KEY_NAME_MAP_RAINFOREST then
            mapType = 3;
        end

        hasDetectMap = mapType > 0;

        if hasDetectMap == true and battleNoticeData ~= nil  and battleNoticeData.ringView ~= nil then
            battleNoticeData.ringView:setGameMapType(mapType);
        end
    end

    return mapType;
end

BattleNoticeDataClass = {
}

function BattleNoticeDataClass:new()
    local object = {
        ringView = nil,--战斗提醒圆环view object
        ringViewShouldShow = false,
        ringViewShowing = false,
        lastDetectMapTime = 0,--上次检测地图时间
        userAngle = -1, --当前用户方向
        detectObjects = {}, --图片上出现的敌人
    }

    setmetatable(object, self);
    self.__index = self;

    return object;
end

--地图上每个物体要生成一个id：针对一个物体是同一个类型，且角度相差只有N度时，认为是同一个物体（不同的帧检测）
local nextObjectId__ = 0;
function BattleNoticeDataClass:generateDedectObjectId()
    nextObjectId__ = nextObjectId__ + 1;
    return nextObjectId__;
end

--判断该物体是否在已有的物体里,角度范围为2度
function BattleNoticeDataClass:getSeeminglyObjects(object)
    return self:getSeeminglyObjectsByParams(self.detectObjects, object, OBJECT_ANGLE_LIMIT);
end

--获取两物体的夹角
local function getTwoObjectIntersectionAngle(object1, object2)
    local angle = math.abs(object1.angle - object2.angle);
    angle = math.min(angle, 360 - angle);

    return angle;
end

--从objects 中得到与object疑似为同一个物体的所有对象，当两物体类型一样且角度在angle范围内，认为是相似物体
function BattleNoticeDataClass:getSeeminglyObjectsByParams(objects, object, angle)
    local seeminglyObjects = {};
    if object == nil or objects == nil then
        return seeminglyObjects;
    end

    for key, value in pairs(objects) do
        if value.type == object.type and
                getTwoObjectIntersectionAngle(object, value) <= angle then
            seeminglyObjects[#seeminglyObjects + 1] = value;
        end
    end

    return seeminglyObjects;
end

function BattleNoticeDataClass:isUserAngleChanged(nowAngle)
    local perAngle = self.userAngle;
    if perAngle < 0 and nowAngle < 0 then
        return false;
    end

    if (perAngle < 0 and nowAngle >= 0) or (perAngle >= 0 and nowAngle < 0) then
        return true;
    end

    if math.abs(perAngle - nowAngle) <= USER_ANGLE_LIMIT then
        return false;
    end

    return true;
end

--上报
function BattleNoticeDataClass:reportRadarTag(usrAngle, soundType, soundAngle, soundSaturation, soundBrightness)
    -- angle:int sound:byte saturation:int brightness:int sound_angle:byte game_map:byte
    commonlib_report(GAMEMASTER_PUBGM_RADAR_TAG_TABLE,
            {
                ["angle"]  = usrAngle,
                ["sound"]  = soundType,
                ["saturation"]  = soundSaturation,
                ["brightness"]  = soundBrightness,
                ["sound_angle"]  = soundAngle,
                ["game_map"]  = getMapType()
            });
end

local function toObjectTypeString(type)
    if type == nil then
        return "nil"

    elseif type == 1 then
        return "脚印"
    elseif type == 2 then
        return "载具"
    elseif type == 3 then
        return "枪声"
    elseif type == 4 then
        return "有消声器的枪声"
    elseif type == 5 then
        return "有玻璃声"
    else
        return "未知"
    end
end

function BattleNoticeDataClass:reportObj(userAngle, detectObjects, newObjects)
    if nil ~= newObjects then
        for key, value in pairs(newObjects) do
            -- value.type, value.angle, value.ave_s, value.ave_v
            self:reportRadarTag(userAngle, value.type, value.angle, value.ave_s, value.ave_v);
            printLog("BattleNoticeDataClass:update:reportRadarTag usrAngle="..tostring(userAngle)..", sound="..toObjectTypeString(value.type));
        end
    end
end

--更新数据
function BattleNoticeDataClass:update(userAngle, detectObjects)
    local addedObjects;
    local removeObjects;
    local newObjects;

    --角度-1时，不处理
    local recognizeUserAngle = userAngle
    if userAngle < 0 then
        userAngle = self.userAngle;
    end

    --printLog("BattleNoticeDataClass:update:old:self.userAngle="..tostring(userAngle)..",self.detectObjects="..self:detectObjectToString(self.detectObjects));
    --printLog("BattleNoticeDataClass:update:detectObjects="..self:detectObjectToString(detectObjects));

    if userAngle >= 0 then
        addedObjects, removeObjects, newObjects = self:mergeDetectObjects(detectObjects);
    else
        addedObjects, removeObjects, newObjects = self:mergeDetectObjects({});
    end

    --printLog("BattleNoticeDataClass:update:addedObjects="..self:detectObjectToString(addedObjects));
    -- -- printLog("BattleNoticeDataClass:update:removeObjects="..self:detectObjectToString(removeObjects));
    local userAngleChanged = false;
    if self:isUserAngleChanged(userAngle) then
        self.userAngle = userAngle;
        userAngleChanged = true;
    end

    self:reportObj(recognizeUserAngle, detectObjects, newObjects)

    --printLog("BattleNoticeDataClass:update:new:self.userAngle="..tostring(userAngle)..",self.detectObjects="..self:detectObjectToString(self.detectObjects));

    return userAngleChanged, addedObjects, removeObjects;
end

----------------------------------------
-- #### 测试代码 开始 ####
local userTipInfo = {
    userDirectiontipView = nil,
    objectDetectView = nil,
}


local TOAST_SHOW_POSITION = {
    anchor = Anchor().TOP,
    x = -50,
    y = 80,
};

local function showUserDirectionTipView(text)
    if text == nil then
        text = ""
    end

    commonlib_runOnUIThread(function()
        if userTipInfo.userDirectiontipView == nil then
            userTipInfo.userDirectiontipView = floatWindow:addTextView(text,
                "#FFFF0000",
                "16sp",
                TOAST_SHOW_POSITION.anchor,
                TOAST_SHOW_POSITION.x,
                TOAST_SHOW_POSITION.y,
                2);
        else
            userTipInfo.userDirectiontipView:setText(text);
        end
    end);
end

local function showObjectDectectTipView(text)
    if text == nil then
        text = ""
    end

    commonlib_runOnUIThread(function()
        if userTipInfo.objectDetectView == nil then
            userTipInfo.objectDetectView = floatWindow:addTextView(text,
                "#FFFF0000",
                "16sp",
                TOAST_SHOW_POSITION.anchor,
                TOAST_SHOW_POSITION.x,
                TOAST_SHOW_POSITION.y + 40,
                2);
        else
            userTipInfo.objectDetectView:setText(text);
        end
    end);
end

local function showTipTextView(bShow)
    commonlib_runOnUIThread(function()

        if userTipInfo.objectDetectView ~= nil then
            if bShow == true then
                userTipInfo.objectDetectView:show();
            else
                userTipInfo.objectDetectView:hide();
            end
        end

        if userTipInfo.userDirectiontipView ~= nil then
            if bShow == true then
                userTipInfo.userDirectiontipView:show();
            else
                userTipInfo.userDirectiontipView:hide();
            end
        end
    end);
end

local function detectObjectToString(detectObjets)
    if detectObjets == nil then
        return "count is 0";
    end

    local retStr = "";
    for key, value in pairs(detectObjets) do
        retStr = retStr.."id="..tostring(value.id)..",type="..tostring(value.type)..",angle="..tostring(value.angle)..";";
    end

    return retStr;
end
-- ##### 测试代码 结束 #####
---------------------------------------------

local function getDelayDeleteTimeMS(type)
    if type == nil then
        return nil

    elseif type == 1 then
        return FOOT_DELAY_TIME --"脚印"，不延迟
    elseif type == 2 then
        return CARRIER_DELAY_TIME --"载具"，不支持
    elseif type == 3 then
        return GUN_DELAY_TIME --"枪声"，延迟2s
    elseif type == 4 then
        return MUFFLER_DELAY_TIME --"有消声器的枪声"，延迟2s
    elseif type == 5 then
        return GLASS_DELAY_TIME -- "玻璃"，不延迟
    else
        return nil --"未知"，不支持
    end
end

local function checkTimeInvalid(time, curTime)
    return (time ~= nil and time > 0 and time <= curTime);
end

local function objectToString(objData)
    if nil == objData then
        return "error data"
    end

    return "id_" .. tostring(objData.id) .. ", type:" .. toObjectTypeString(objData.type) .. ", angle:" .. tostring(objData.angle)
            .. ", st:" ..  tostring(objData.startTime) .. ", et:" .. tostring(objData.endTime)
end

local function checkDelete(objData)
    if nil == objData then
        return true
    end

    local delayTime = getDelayDeleteTimeMS(objData.type)
    if delayTime == nil or delayTime <= 0 then
        printLog("checkDelete return true, not delay, ".. objectToString(objData))
        return true
    end

    local lCurTimeMS = commonlib_getCurrentTime()
    if not checkTimeInvalid(objData.startTime, lCurTimeMS) or not checkTimeInvalid(objData.endTime, lCurTimeMS) or objData.startTime > objData.endTime then
        printLog("checkDelete return true, time args error, ".. objectToString(objData))
        return true
    end

    if (objData.endTime - objData.startTime) <= OBJECT_END_TIME_LIMIT then
        -- 小于或等于真正消失的时间，认为物体转移，也是直接删除
        printLog("checkDelete return true, obj move, ".. objectToString(objData))
        return true
    end

    local bRet = (lCurTimeMS - objData.endTime >= delayTime)
    printLog("checkDelete return ".. tostring(bRet) .. ", delaytime:" .. tostring(delayTime) .. ", " .. objectToString(objData))
    return bRet
end

local function getObjectAlpha(type, ave_s, ave_v)
    local alphaValue = 1.0
    if nil ~= ave_s and nil ~= ave_v and (ave_s <= AVE_S_MAX or ave_v <= AVE_V_MAX) then
        alphaValue = DISTANT_OBJECT_ALPHA
    end
    printLog("getObjectAlpha return ".. tostring(alphaValue) .. ", " .. toObjectTypeString(type)
            .. ", ave_s:" .. tostring(ave_s) .. ", ave_v:" .. tostring(ave_v))
    return alphaValue
end

--合并检测的物体
--最新帧检测的物体与上一帧数据合并：得到新增的object，及需要删除的object
function BattleNoticeDataClass:mergeDetectObjects(detectObjets)
    local newObjects = {};
    local addedObjects = {};
    local removeObjects = {};
    local stillExsitObjects = {};

    local lCurTime = commonlib_getCurrentTime();

    if detectObjets == nil then
        detectObjets = {};
    end

    if self.detectObjects == nil then
        self.detectObjects = {};
    end

    for key, value in pairs(detectObjets) do
        value =
        {
            id = nil,
            type = value.type,
            angle = value.angle,
            ave_s = value.ave_s,
            ave_v = value.ave_v,
            alpha = getObjectAlpha(value.type, value.ave_s, value.ave_v),
            startTime = nil,
            endTime = nil,
            disappear = false,
        };

        local seeminglyObjects = self:getSeeminglyObjects(value);
        --printLog("BattleNoticeDataClass:mergeDetectObjects:seeminglyObjects="..self:detectObjectToString(seeminglyObjects));
        local notExsitInStillExsitObjectsArray = {};

        if seeminglyObjects ~= nil and #seeminglyObjects > 0 then
            --找出：不在 stillExsitObjects数组里的seeminglyObjects中所有的元素
            --存在notExsitInStillExsitObjectsArray中
            --当notExsitInStillExsitObjectsArray为空，说明新增元素
            --不为空，认为notExsitInStillExsitObjectsArray中第一个元素与value是同一个物体
            for keyS, valueS in pairs(seeminglyObjects) do
                if IsInTable(stillExsitObjects, valueS) == false then
                    notExsitInStillExsitObjectsArray[#notExsitInStillExsitObjectsArray + 1] = valueS;
                end
            end
        end

        if #notExsitInStillExsitObjectsArray == 0 then
            --表示新增的物体
            value.id = self:generateDedectObjectId();
            value.startTime = lCurTime;
            value.endTime   = nil;
            addedObjects[#addedObjects + 1] = value;
            newObjects[#newObjects+1] = value;

            printLog("New_Obj, id_".. tostring(value.id) .. ", type:" .. toObjectTypeString(value.type) .. ", angle:" .. tostring(value.angle))
        else
            --存在相同的物体，认为是已有的物体

            local stillExistObj = notExsitInStillExsitObjectsArray[1];
            if nil ~= stillExistObj and nil ~= stillExistObj.endTime then
                stillExistObj.startTime = lCurTime; -- 且从无到有，标为开始
                stillExistObj.endTime   = nil;      -- 此处认为未结束

                printLog("Reset_Obj, id_".. tostring(stillExistObj.id) .. ", type:" .. toObjectTypeString(stillExistObj.type) .. ", angle:" .. tostring(stillExistObj.angle))
            end

            stillExsitObjects[#stillExsitObjects + 1] = stillExistObj;
        end
    end

    for key, value in pairs(self.detectObjects) do
        if IsInTable(stillExsitObjects, value) == false then

            -- 物体首次消失设置结束时间
            if value.endTime == nil then
                value.endTime = lCurTime;
            end

            if checkDelete(value) then
                removeObjects[#removeObjects + 1] = value;
            else
                -- 延迟删除的物体，继续添加到增加物体列表上
                value.disappear = true;
                addedObjects[#addedObjects + 1] = value;

                printLog("Disappear_Obj, id_".. tostring(value.id) .. ", type:" .. toObjectTypeString(value.type) .. ", angle:" .. tostring(value.angle))
            end
        end
    end

    self.detectObjects = stillExsitObjects;
    for key, value in pairs(addedObjects) do
        self.detectObjects[#self.detectObjects + 1] = value;
    end

    return addedObjects, removeObjects, newObjects;
end

local function updateRingView(userAngle, addedObjects, removeObjects)

    printLog("updateRingView:userAngle="..tostring(userAngle));
    -- 测试代码
    if TIP_SHOW_ENABLE then
        printLog("updateRingView:addedObjects="..detectObjectToString(addedObjects));
        printLog("updateRingView:update:removeObjects="..detectObjectToString(removeObjects));
    end

    if battleNoticeData.ringView == nil then
        return;
    end

    commonlib_runOnUIThread(function()
        printLog("really ringView:update, userAngle="..tostring(userAngle));
        battleNoticeData.ringView:update(userAngle, addedObjects, removeObjects);
    end);
end

local function checkNeedMoveRingView(tag)
    if nil == battleNoticeData or not battleNoticeData.ringViewShowing then
--        printLog("moveRingView, ringView is not showing!!!" .. tag)
        return false
    end

    if hasInCardOrShip == mLastRingInHorizontalCenter then
--        printLog("moveRingView, pos is not changed!!!" .. tag)
        return false
    end

    return true
end

local function moveRingView()
    if not checkNeedMoveRingView("WorkThread") then
        return
    end

    commonlib_runOnUIThread(function()
        if not checkNeedMoveRingView("UiThread") then
            return
        end

        local bRet = false
        if hasInCardOrShip then
            bRet = battleNoticeData.ringView:moveToHorizontalCenter()
            printLog("moveRingView, moveToHorizontalCenter() return " .. tostring(bRet))
        else
            bRet = battleNoticeData.ringView:restoreBottomOvalPoint()
            printLog("moveRingView, restoreBottomOvalPoint() return " .. tostring(bRet))
        end

        if bRet then
            mLastRingInHorizontalCenter = hasInCardOrShip
        end
    end)
end

local function reportAssistShowError()
    notifyAssistShowError(ASSIST_SHOW_TYPE_RADAR_CICLE, gRadarSwitch)
    notifyAssistShowError(ASSIST_SHOW_TYPE_RADAR_TOAST, gRemindSwitch)
end

local function reportAssistShowSucceed()
    notifyAssistShowSucceed(ASSIST_SHOW_TYPE_RADAR_RING)

    if gRadarSwitch then
        notifyAssistShowSucceed(ASSIST_SHOW_TYPE_RADAR_CICLE)
    else
        notifyAssistShowError(ASSIST_SHOW_TYPE_RADAR_CICLE, false)
    end

    if gRemindSwitch then
        notifyAssistShowSucceed(ASSIST_SHOW_TYPE_RADAR_TOAST)
    else
        notifyAssistShowError(ASSIST_SHOW_TYPE_RADAR_TOAST, false)
    end
end

local function showRingView(bShow)

    printLog("showRingView:" .. tostring(bShow) .. ", shouldshow:" .. tostring(battleNoticeData.ringViewShouldShow))

    if TIP_SHOW_ENABLE then
        showTipTextView(bShow); -- 测试代码
    end

    if battleNoticeData.ringViewShouldShow == bShow then

        if not bShow then
            reportAssistShowError()
        end

        return;
    end

    battleNoticeData.ringViewShouldShow = bShow;

        commonlib_runOnUIThread(function()

            if battleNoticeData.ringView ~= nil and
                    battleNoticeData.ringViewShouldShow ~= battleNoticeData.ringViewShowing then
                battleNoticeData.ringViewShowing = battleNoticeData.ringViewShouldShow;
                if battleNoticeData.ringViewShowing then
                    printLog("showRingView, ringView.show()")
                    battleNoticeData.ringView:show();
                    reportAssistShowSucceed()
                else
                    battleNoticeData.ringView:hide();
                    mLastRingInHorizontalCenter = false;
                    printLog("showRingView, ringView.hide()")
                    reportAssistShowError()
                end
            end
    end);
end

local function getRingViewSwtich()
    if gRadarSwitch == true and gRemindSwitch == true then
        return SHOW_ALL;
    elseif gRadarSwitch == true and gRemindSwitch == false then
        return SHOW_RADAR;
    elseif gRadarSwitch == false and gRemindSwitch == true then
        return SHOW_REMIND_TIPS;
    else
        return NOT_SHOW_ALL;
    end
    return NOT_SHOW_ALL;
end

local function updateRingViewSwitch()

    local toUpdateRingViewSwitch = getRingViewSwtich();
    if currentRingViewSwitch == toUpdateRingViewSwitch then
        return
    end

    commonlib_runOnUIThread(function()
        if currentRingViewSwitch == toUpdateRingViewSwitch then
            return
        end

        currentRingViewSwitch = toUpdateRingViewSwitch;
        if battleNoticeData.ringView ~= nil then
            battleNoticeData.ringView:setSwitch(currentRingViewSwitch);
        end
    end);
end

local function withinDiffTimeAirportEndSysTime(howlong)
    local glAirportEndSysTime = commonlib_getGameParam(KEY_AIRPORT_END_SYS_TIME)
    -- 识别到机场倒计时，并且在机场倒计时为0时到AFTER_AIRPORT_END_SYS_TIME_DETECT_INFO后识别INFO
    if glAirportEndSysTime ~= nil and glAirportEndSysTime > -2000 then
        local difftime = commonlib_getCurrentTime() - glAirportEndSysTime
        if difftime  < howlong and difftime > 0 then
            printLog("withinDiffTimeAirportEndSysTime, difftime=" .. tostring(difftime) .. ", howlong=" .. tostring(howlong))
            return true
        end
    end
    return false
end

local function updateDetectInfos()
    if withinDiffTimeAirportEndSysTime(AFTER_AIRPORT_END_SYS_TIME_DETECT_INFO) then
        return
    end

    local userDirction = gNewGameEngine:matchConditionOnNumber("cd_build_cj_detect_user_direction");
    if userDirction == nil then
        userDirction = -1
    end

    local objects = gNewGameEngine:matchConditionOnTable("cd_build_cjdetect_objects");
    if objects == nil then
        objects = {}
    end

   -- if userDirction < 0 then
    --    commonlib_saveImage(mapRect, "UserDirection_"..tostring(commonlib_getCurrentTime()))
    --end
    ---------------------------------
    -- #### 测试代码 开始 ####
    if TIP_SHOW_ENABLE then
        showUserDirectionTipView("用户方向："..tostring(userDirction).."度");
        if objects == nil or #objects == 0 then
            showObjectDectectTipView("地图物体识别个数为0");
        else
            local textShow = "识别到物体: ";
            for i, value_ in pairs(objects) do
                textShow = textShow..tostring(value_["angle"])..","..toObjectTypeString(value_["type"])..";";
                -- textShow = textShow .. toObjectTypeString(value_["type"]) .. "[" .. tostring(value_["ave_s"])..","..tostring(value_["ave_v"]).."];";
            end
            showObjectDectectTipView(textShow);
        end
    end
    -- #### 测试代码 结束 ####
    -----------------------------------

    local userAngleChanged, addedObjects, removeObjects = battleNoticeData:update(userDirction, objects);
    if userAngleChanged == true or #addedObjects > 0 or #removeObjects > 0 then
        updateRingView(battleNoticeData.userAngle, addedObjects, removeObjects);
    end
end

local function canShowRingViewImpl()
    return getRingViewSwtich() ~= NOT_SHOW_ALL
end

local function canShowRingView()
    if withinDiffTimeAirportEndSysTime(AFTER_AIRPORT_END_SYS_TIME_SHOW_RING_VIEW) then
        return false
    end

    if not canShowRingViewImpl() then
		return false
	end
    local hasSightMode = commonlib_getGameParam(KEY_IS_SIGHT_MODE);
    return hasSightMode ~= nil and hasSightMode == false
end


local function uploadBitmap()
    if uploadBitmapCount >= MAX_UPLOAD_BITMAP_COUNT then
        return
    end

    local currentTime = commonlib_getCurrentTime()
    if currentTime - enterHomeTime <= 0 or enterHomeTime == 0 then
        return
    end

    if currentTime - enterHomeTime < 2 * 60 * 1000 then
        return
    end

    local rects = gNewGameEngine:getConditionRect("cd_build_rect_map_rect")
    if rects == nil then
        return
    end
    local filename = "pubgmhd_smallmap_" .. tostring(currentTime)
    commonlib_getUtilTool():uploadbitmap(rects, filename, SHOT_SCREEN_FOR_MAT_TYPE.pubgmhd_smallmap)
    uploadBitmapCount = uploadBitmapCount + 1
    printLog("ScreenshotDataForMat uploadBitmap: filename=" .. tostring(filename))
end

local function onEnterGame()
    printLog("onEnterGame")
    hasInCardOrShip = false;
    enterHomeTime = commonlib_getCurrentTime()
    mLastRingInHorizontalCenter = false;
    showRingView(canShowRingView());
end

local function onGameRunning()
    local cover = commonlib_getGameParam(KEY_GAME_RUNNING_COVER_STATUS);
    if cover ~= GAME_COVER_NONE then
        showRingView(false);
        return;
    end

    --uploadBitmap()

    local running = commonlib_getGameParam(KEY_GAME_RUNNING_SUBSTATE);
    hasInCardOrShip = (running == PUBGMHD_GAMESTATE_RUNNING_CAR_SHIP);

    printLog("onGameRunning:start,hasInCardOrShip="..tostring(hasInCardOrShip))
    if canShowRingView() then
        updateRingViewSwitch();
        showRingView(true)
        moveRingView();
        updateDetectInfos();
    else
        showRingView(false)
    end
    printLog("onGameRunning:end")
end

local function onGameOver()
    showRingView(false);
    hasDetectMap = false;
end

local function initVariable()

    gNewGameEngine = CNewGameEngine:getInstance()

    floatWindow = commonlib_getUnClickableFloatView();
    battleNoticeData = BattleNoticeDataClass:new();
    gRecognizeAdapterManager = AdapterManagerClass:getRecognizeInstance();
    local itemRect = gNewGameEngine:getConditionRect(KEY_NAME_BN_RING_VIEW_CENTER_POSITION);
    if itemRect == nil then
        printLog("RecognizeAdapterManager:getItem(KEY_NAME_BN_RING_VIEW_CENTER_POSITION) nil")
    end

    --创建ringView
    battleNoticeData.ringView = floatWindow:addRingView(
            Anchor().TOPLEFT, 0, 0, ZOrder().zorder_bottom)

    battleNoticeData.ringView:initPosition(itemRect.flag, itemRect.left, itemRect.top)
end


local function unInitVariable()

    if battleNoticeData ~= nil and battleNoticeData.ringView ~= nil then
        battleNoticeData.ringView:destroy();
    end

    floatWindow = nil
    battleNoticeData = nil;
end


--- 设置面板相关
local function initConfigUI()
    printLog("initConfigUI begin")
end

local function initReportTable()
end

local function getTextColor(enable)
    if enable == true then
        return "#FF00DEFF";
    else
        return "#FF979797";
    end
end

local function updateItemDescTextColor(textView, isChecked)
    if textView ~= nil then
        textView:setTextColor(getTextColor(isChecked))
    end
end

local function updateTotalCheckBoxState()
    if gRadarSwitch == false and gRemindSwitch == false then

    end
end

local function onRadarStateChangedListener(checkBoxId, isChecked)
    commonlib_setSharedPrefBoolean(SP_SOUND_RADAR_SWITCH_KEY, isChecked)
    gRadarSwitch = isChecked

    if isChecked then
        if nil ~= battleNoticeData and nil ~= battleNoticeData.ringView and battleNoticeData.ringViewShowing == true then
            notifyAssistShowSucceed(ASSIST_SHOW_TYPE_RADAR_CICLE)
        else
            notifyAssistSwitchToOpen(ASSIST_SHOW_TYPE_RADAR_CICLE)
        end

        if nil ~= gRadarCheckBoxRedDot then
            gRadarCheckBoxRedDot:hide()
        end
    end

    updateTotalCheckBoxState()
    updateItemDescTextColor(gRadarDescTextView, isChecked)
end

local function onRemindStateChangedListener(checkBoxId, isChecked)
    commonlib_setSharedPrefBoolean(SP_REMIND_SWITCH_KEY, isChecked)
    gRemindSwitch = isChecked

    if isChecked then
        if nil ~= battleNoticeData and nil ~= battleNoticeData.ringView and battleNoticeData.ringViewShowing == true then
            notifyAssistShowSucceed(ASSIST_SHOW_TYPE_RADAR_TOAST)
        else
            notifyAssistSwitchToOpen(ASSIST_SHOW_TYPE_RADAR_TOAST)
        end
    end

    updateTotalCheckBoxState()
    updateItemDescTextColor(gRemindDescTextView, isChecked)
end

local function initSettingView()
    local settingView = commonlib_getSettingView()
    local tab = settingView:addTableView(TAB_NAME)

    local relativeLayoutParam = create("RelativeLayoutParam")
    local paramValues = {}
    paramValues.layout_marginTop = "20dp"
    paramValues.layout_marginRight = "40dp"
    paramValues.layout_below = nil
    paramValues.layout_marginLeft = "40dp"
    relativeLayoutParam:setLayoutParam(paramValues)
    gRadarSwitch = getSoundRadarSwitchState()
    gRadarCheckBox = tab:addCheckBoxView(RADAR_CHECKBOX_TEXT, gRadarSwitch, relativeLayoutParam, onRadarStateChangedListener)
    if gRadarSwitch then
        report_gamemaster_game_pubgm_state(1, 1)
    else
        report_gamemaster_game_pubgm_state(1, 2)
    end

    paramValues.layout_below = nil
    paramValues.layout_marginLeft = "12dp"
    paramValues.layout_marginTop = "23.5dp"
    paramValues.layout_width = "23.27dp"
    paramValues.layout_height = "10.67dp"
    relativeLayoutParam:setLayoutParam(paramValues)
    gRadarCheckBoxRedDot = tab:addImageView("img/setting_hot.png", relativeLayoutParam, nil)
    gRadarCheckBoxRedDot:hide()

    paramValues.layout_below = gRadarCheckBox:getId()
    paramValues.layout_marginLeft = "65dp"
    paramValues.layout_marginTop = "12dp"
    paramValues.layout_width = "95dp"
    paramValues.layout_height = "42dp"
    relativeLayoutParam:setLayoutParam(paramValues)
    local radarImg = tab:addImageView("img/setting_sound_radar.png", relativeLayoutParam, nil)

    paramValues.layout_below = radarImg:getId()
    paramValues.layout_marginTop = "10dp"
    paramValues.layout_width = "WRAP_CONTENT"
    paramValues.layout_height = "WRAP_CONTENT"
    relativeLayoutParam:setLayoutParam(paramValues)
    gRadarDescTextView = tab:addTextView(RADAR_DESC_TEXT,
            getTextColor(gRadarSwitch),
            "12sp",
            relativeLayoutParam,
            nil);


    paramValues.layout_below = gRadarDescTextView:getId()
    paramValues.layout_marginLeft = "40dp"
    relativeLayoutParam:setLayoutParam(paramValues)
    gRemindSwitch = getRemindSwitchState()
    gRemindCheckBox = tab:addCheckBoxView(REMIND_CHECKBOX_TEXT, gRemindSwitch, relativeLayoutParam, onRemindStateChangedListener)
    if gRemindSwitch then
        report_gamemaster_game_pubgm_state(2, 1)
    else
        report_gamemaster_game_pubgm_state(2, 2)
    end
    paramValues.layout_below = gRemindCheckBox:getId()
    paramValues.layout_marginLeft = "65dp"
    paramValues.layout_marginTop = "12dp"
    paramValues.layout_width = "193dp"
    paramValues.layout_height = "30dp"
    relativeLayoutParam:setLayoutParam(paramValues)
    local remindImg = tab:addImageView("img/setting_remind.png", relativeLayoutParam, nil)

    paramValues.layout_below = remindImg:getId()
    paramValues.layout_marginTop = "10dp"
    paramValues.layout_width = "WRAP_CONTENT"
    paramValues.layout_height = "WRAP_CONTENT"
    relativeLayoutParam:setLayoutParam(paramValues)
    gRemindDescTextView = tab:addTextView(REMIND_DESC_TEXT,
            getTextColor(gRemindSwitch),
            "12sp",
            relativeLayoutParam,
            nil);

    currentRingViewSwitch = getRingViewSwtich();
    if currentRingViewSwitch ~= SHOW_ALL then
        battleNoticeData.ringView:setSwitch(currentRingViewSwitch);
    end
end

local function onSettingViewOpen()
    local redDot = commonlib_getGameParam(KEY_IS_SHOW_RED_DOT)
    printLog("RedDot, onSettingViewOpen, gRadarSwitch=" .. tostring(gRadarSwitch) .. ", redDot=" .. tostring(redDot))
    if redDot and not gRadarSwitch then
        gRadarCheckBoxRedDot:show()
    else
        gRadarCheckBoxRedDot:hide()
    end
end

local function onSettingViewClose()
    printLog("onSettingViewClose")
    local redDot = commonlib_getGameParam(KEY_IS_SHOW_RED_DOT)
    if not redDot or gRadarSwitch then
        gRadarCheckBoxRedDot:hide()
    end
end

local function registerGameEvents()
    commonlib_registerCallback(common_game_events_callbacks[KEY_ENTER_GAME], onEnterGame)
    commonlib_registerCallback(common_game_events_callbacks[KEY_GAME_RUNNING], onGameRunning)
    commonlib_registerCallback(common_game_events_callbacks[KEY_GAME_OVER], onGameOver)

    commonlib_registerSettingViewOpenCallback(onSettingViewOpen)
    commonlib_registerSettingViewCloseCallback(onSettingViewClose)
end

local function unregisterGameEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_ENTER_GAME], onEnterGame)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_GAME_RUNNING], onGameRunning)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_GAME_OVER], onGameOver)

    commonlib_unRegisterSettingViewOpenCallback(onSettingViewOpen)
    commonlib_unRegisterSettingViewCloseCallback(onSettingViewClose)
end

local function init()
    printLog("fight init")
    registerGameEvents()
    initVariable()
    initReportTable()
    initConfigUI()
    initSettingView()
end

local function uninit()
    printLog("fight uninit")
    unInitVariable()
    unregisterGameEvents()
end

table.insert(common_life_circle_callbacks, { init, uninit, PUBGMHD_LIFE_CIRCLE_TABLE_ORDER_WEIGHT.battle_notice })