-- 瞄准器脚本
local TAG = "gun_sight"

local function printLog(msg)
    commonlib_print(TAG, msg)
end

local function canShowGunSight()
    if common_gun_sight_getSettingViewOpenState() then
        return false
    end

    local weapon = commonlib_getGameParam(KEY_WEAPON_ICON);
    if weapon == WEAPON_NONE then
        return false;
    end

    local running = commonlib_getGameParam(KEY_GAME_RUNNING_SUBSTATE);
    if running ~= PUBGMHD_GAMESTATE_RUNNING_SHOOT then
        return false;
    end

    local hasSightMode = commonlib_getGameParam(KEY_IS_SIGHT_MODE);
    if hasSightMode ~= nil and hasSightMode then
        return false
    end

    return true;
end

local function showGunSight()
    local isCbSwitchChecked = common_gun_sight_getCbSwitchCheckState()
    local isCbAlwaysOpenChecked = common_gun_sight_getCbAlwaysOpenCheckState()
    commonlib_runOnUIThread(function()
        common_gun_sight_updateGunSight();
        local cover = commonlib_getGameParam(KEY_GAME_RUNNING_COVER_STATUS);
        if not isCbSwitchChecked or cover ~= GAME_COVER_NONE then
            printLog("Checkbox未选中或遮挡状态，隐藏瞄准器")
            common_gun_sight_show(false);
        elseif isCbAlwaysOpenChecked or canShowGunSight() then
            printLog("显示瞄准器")
            common_gun_sight_show(true);
        else
            printLog("隐藏瞄准器")
            common_gun_sight_show(false);
        end
    end)
end

local function onGameRunning()
    showGunSight();
end

local function onGameOver()
    printLog("onGameOver")
    common_gun_sight_showOnUiThread(false);
end

local function registerEvents()
    commonlib_registerCallback(common_game_events_callbacks[KEY_GAME_RUNNING], onGameRunning)
    commonlib_registerCallback(common_game_events_callbacks[KEY_GAME_OVER], onGameOver)
    printLog("事件注册成功")
end

local function unRegisterEvents()
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_GAME_RUNNING], onGameRunning)
    commonlib_unregisterCallback(common_game_events_callbacks[KEY_GAME_OVER], onGameOver)
end

local function init()
    printLog("gun_sight script init")
    common_gun_sight_init()
    common_gun_sight_show(false)
    registerEvents();
end

local function uninit()
    printLog("gun_sight script uninit")
    common_gun_sight_uninit()
    unRegisterEvents();
end

table.insert(common_life_circle_callbacks, { init, uninit, PUBGMHD_LIFE_CIRCLE_TABLE_ORDER_WEIGHT.gun_sight });