---
--- Generated by EmmyLua(https://github.com/EmmyLua)
--- Created by jinghao.
--- DateTime: 2018/5/28 10:55
---

local TAG = "keyboard_tips"

local function printLog(msg)
    commonlib_print(TAG, msg)
end

local keyboardModule
local gRecognizeAdapterManager;

local function initKeyboardModule()
    keyboardModule = KeyboardModule()
    gRecognizeAdapterManager = AdapterManagerClass:getRecognizeInstance()
end

--- 检测当前是否已开启游戏超人键盘
local function isSetKeyboardAsDefault()
    local res = keyboardModule:isGameMasterKeyboardAsDefault(null)
    printLog("isOpenSuperMasterKeybaord " .. tostring(res))
    return res
end

local function isAddKeyboardToSystem()
    local res = keyboardModule:isAddGameMaterKeyboardToSystem(null)
    printLog("isAddKeyboardToSystem " .. tostring(res))
    return res
end

local function isEnableKeyboardLocal()
    local res = keyboardModule:isEnableLocal()
    printLog("isEnableKeyboardLocal " .. tostring(res))
    return res
end

local function isEnableKeyboardCloud()
    local res = keyboardModule:isEnable()
    printLog("isEnableKeyboardCloud " .. tostring(res))
    return res
end

local function isEnableKeyboard()
    if isEnableKeyboardCloud() and isEnableKeyboardLocal() and isAddKeyboardToSystem() then
        return true
    end
    return false
end

local tipsFloatView
local tipsView
local ACTION_CLICK = "action_click"
local curIsFullCover
local isKeyboardEnable
local isSetDefault
local fullCoverTime

local function dismissTips()
    commonlib_runOnUIThread(function ()
        printLog("dismiss PlayWindowOnUIThread")
        if tipsFloatView ~= nil then
            tipsFloatView:destroy()
            tipsFloatView = nil
        end
        if tipsView ~= nil then
            tipsView:destroy()
            tipsView = nil
        end
    end)
end

local function addTipsView(offsetY)
    commonlib_runOnUIThread(function ()
        printLog("addTipsView ："  .. tostring(offsetY / commonlib_getScale()))
        if tipsFloatView == nil then
            tipsFloatView = commonlib_getFloatViewMgrEx():create(
                    0, ((offsetY  - commonlib_dp2Pix(26)) / commonlib_getScale()),
                    COMMON_LAYOUTPARAM.WRAP_CONTENT, COMMON_LAYOUTPARAM.WRAP_CONTENT,
                    Anchor().TOPLEFT,
                    true);
            if tipsView == nil then
                tipsView = tipsFloatView:addKeyboardTipsView(Anchor().TOPLEFT, 0, 0, ZOrder().zorder_top);
                tipsView:setOnEventListener(function (action)
                    printLog("on event ："  .. action)
                    if action == ACTION_CLICK then
                        dismissTips()
                        --elseif action == ACTION_CLICK_PREVIEW then
                        --elseif action == ACTION_CLICK_REPLAY then
                        --elseif action == ACTION_START_PLAY then
                        --elseif action == ACTION_PLAY_COMPLETE then
                        keyboardModule:switchSystemKeyboard(null)
                    end
                end)
            else
                printLog("tips view not null, check for reason")
            end
            tipsFloatView:show();
        else
            printLog("tips float view not null, check for reason")
        end
    end)
end

local function handleShowTips()
    if curIsFullCover == false then
        --printLog("return : not full cover")
        return
    end
    local currentTime = commonlib_getCurrentTime()
    if fullCoverTime == nil or currentTime - fullCoverTime < 1000 or currentTime - fullCoverTime > 5000 then
        printLog("return : time check fail ")
        printLog("currentTime : " .. tostring(currentTime))
        printLog("fullCoverTime : " .. tostring(fullCoverTime))
        return
    end
    if isSetDefault == true then
        printLog("return : is set keyboard as default")
        return
    end
    if isKeyboardEnable == false then
        printLog("return : keyboard disable")
        return
    end
    if tipsFloatView ~= nil then
        printLog("return : tips is showing")
        return
    end
    local offsetY = gRecognizeAdapterManager:doRecognizeByKeyOnIntResult(KEY_KB_DETECT_METHOD_OPENED_REG_INFO);
    printLog("offsetY : " .. tostring(offsetY))
    if offsetY > 0 then
        addTipsView(offsetY)
    end
end

local function onGameRunning()
    if commonlib_isFlameMountainMap() then
        printLog("onGameRunning unsupport FlameMountainMap")
        return;
    end
    --printLog("on game running " .. tostring(curIsFullCover))
    handleShowTips()
end

local function onFullCover(isFullCover)
    printLog("FullCover state=>" .. tostring(isFullCover))
    curIsFullCover = isFullCover
    if isFullCover then
        fullCoverTime = commonlib_getCurrentTime() --- 更新全遮挡时间
        if isKeyboardEnable == nil then
            isKeyboardEnable = isEnableKeyboard() --- 是否开启超人键盘
        end
        isSetDefault = isSetKeyboardAsDefault() --- 是否已设置为默认键盘
    else
        fullCoverTime = 0
        dismissTips()
    end
end

local function onGameOver()
    dismissTips()
    isKeyboardEnable = nil
    isSetDefault = nil
end

local function registerGameEventCallbacks()
    commonlib_registerCallback(common_game_events_callbacks.common_full_cover, onFullCover)
    commonlib_registerCallback(common_game_events_callbacks.common_game_running, onGameRunning)
    commonlib_registerCallback(common_game_events_callbacks.common_game_over, onGameOver)
end

local function unregisterGameEventCallbacks()
    commonlib_unregisterCallback(common_game_events_callbacks.common_full_cover, onFullCover)
    commonlib_unregisterCallback(common_game_events_callbacks.common_game_running, onGameRunning)
    commonlib_unregisterCallback(common_game_events_callbacks.common_game_over, onGameOver)
end

local function init()
    --printLog("keyboard init")

    initKeyboardModule()
    registerGameEventCallbacks()
end

local function uninit()
    --printLog("keyboard uninit")
    unregisterGameEventCallbacks()
    dismissTips()
end

--table.insert(common_life_circle_callbacks, { init, uninit })