
-- 瞄准器脚本
local TAG = "common_gun_sight"
local function printLog(msg)
    commonlib_print(TAG, msg)
end

local GAME_ID
local KEY_IS_SWITCH_CHECKED
local KEY_IS_ALWAYS_OPEN
local KEY_IS_LAST_ALWAYS_OPEN
local KEY_GUN_SIGHT_STYLE
local KEY_GUN_SIGHT_COLOR
local KEY_GUN_SIGHT_SIZE


local TAB_PAGE_NAME = "瞄准器"
local SWITCH_CHECKBOX_TEXT = "开启瞄准器"
local ALWAYS_OPEN_CHECKBOX_TEXT = "始终开启瞄准器"

local GUN_SIGHT_PROPERTY_STYLE = "gunSightStyle";
local GUN_SIGHT_PROPERTY_COLOR = "gunSightColor";
local GUN_SIGHT_PROPERTY_SIZE = "gunSightSize";

local GUN_SIGHT_STYLE_ONE = "one"
local GUN_SIGHT_STYLE_TWO = "two"
local GUN_SIGHT_STYLE_THREE = "three"
local GUN_SIGHT_STYLE_FOUR = "four"
local GUN_SIGHT_STYLE_FIVE = "five"

local GUN_SIGHT_COLOR_WHITE = "white"
local GUN_SIGHT_COLOR_PINK = "pink"
local GUN_SIGHT_COLOR_RED = "red"
local GUN_SIGHT_COLOR_BLUE = "blue"
local GUN_SIGHT_COLOR_GREEN = "green"
local GUN_SIGHT_COLOR_BROWN = "brown"

local GUN_SIGHT_IMG_PATH_DIR = "common://image/gunsight/style/"

local SETTING_VIEW_DOT_PATH = "common://image/gunsight/settingui/dot.png";

local GUN_SIGHT_SELECTED_BG_IMAGE_PATH_TABLE = {
    [GUN_SIGHT_PROPERTY_STYLE] = "common://image/gunsight/settingui/styleIcon/selected.png",
    [GUN_SIGHT_PROPERTY_COLOR] = "common://image/gunsight/settingui/colorIcon/selected.png"
}

local GUN_SIGHT_STYLE_ICON_LIST = {
    [GUN_SIGHT_STYLE_ONE] = "common://image/gunsight/settingui/styleIcon/one.png",
    [GUN_SIGHT_STYLE_TWO] = "common://image/gunsight/settingui/styleIcon/two.png",
    [GUN_SIGHT_STYLE_THREE] = "common://image/gunsight/settingui/styleIcon/three.png",
    [GUN_SIGHT_STYLE_FOUR] = "common://image/gunsight/settingui/styleIcon/four.png",
    [GUN_SIGHT_STYLE_FIVE] = "common://image/gunsight/settingui/styleIcon/five.png"
}

local GUN_SIGHT_COLOR_ICON_LIST = {
    [GUN_SIGHT_COLOR_BLUE] = "common://image/gunsight/settingui/colorIcon/blue.png",
    [GUN_SIGHT_COLOR_BROWN] = "common://image/gunsight/settingui/colorIcon/brown.png",
    [GUN_SIGHT_COLOR_GREEN] = "common://image/gunsight/settingui/colorIcon/green.png",
    [GUN_SIGHT_COLOR_PINK] = "common://image/gunsight/settingui/colorIcon/pink.png",
    [GUN_SIGHT_COLOR_WHITE] = "common://image/gunsight/settingui/colorIcon/white.png",
    [GUN_SIGHT_COLOR_RED] = "common://image/gunsight/settingui/colorIcon/red.png"
}

local GUN_SIGHT_IMAGE_SIZE_SMALL = 94
local GUN_SIGHT_IMAGE_SIZE_BIG = 113

local GUN_SIGHT_SIZE_TEXT_LIST = {
    [GUN_SIGHT_IMAGE_SIZE_SMALL] = "适中",
    [GUN_SIGHT_IMAGE_SIZE_BIG] = "较大"
}

local gunSightProperties = {
    [GUN_SIGHT_PROPERTY_STYLE] = nil,
    [GUN_SIGHT_PROPERTY_COLOR] = nil,
    [GUN_SIGHT_PROPERTY_SIZE] = nil
}
local gunSightNextProperties = {
    [GUN_SIGHT_PROPERTY_STYLE] = GUN_SIGHT_STYLE_ONE,
    [GUN_SIGHT_PROPERTY_COLOR] = GUN_SIGHT_COLOR_GREEN,
    [GUN_SIGHT_PROPERTY_SIZE] = GUN_SIGHT_IMAGE_SIZE_SMALL
}
local GUN_SIGHT_POSITION =  {rect = {flag = Anchor().CENTER, left = 0, top = 0 }}

local appName = 0

local gunSightStyleIvList = {};
local gunSightColorIvList = {};
local gunSightSizeCbList = {};

local switchCheckBoxView = nil
local alwaysOpenCheckBoxView = nil
local isCbSwitchChecked = true
local isCbAlwaysOpenChecked = true
local isCbAlwaysOpenLastChecked = false;

local gunSight = nil;
local gunSightShowing = false;
local floatView
local gbSettingViewOpen

---- should run on ui thread
local function showGunSightImpl(shouldShow)
    if gunSight == nil then
        return;
    end

    if gunSightShowing == shouldShow then
        return;
    end

    gunSightShowing = shouldShow;
    if gunSightShowing then
        gunSight:show();
    else
        gunSight:hide();
    end
end

local function showGunSightOnUiThread(shouldShow)
    commonlib_runOnUIThread(function()
        showGunSightImpl(shouldShow);
    end)
end

local function setWidgetEnable(isEnable)
    printLog("setWidgetEnable : " .. tostring(isEnable))

    alwaysOpenCheckBoxView:setEnabled(isEnable)
    if isEnable then
        alwaysOpenCheckBoxView:setChecked(isCbAlwaysOpenLastChecked)
    else
        isCbAlwaysOpenLastChecked = alwaysOpenCheckBoxView:isChecked();
        commonlib_setSharedPrefBoolean(KEY_IS_LAST_ALWAYS_OPEN, isCbAlwaysOpenLastChecked)
        alwaysOpenCheckBoxView:setChecked(false)
    end

    for key, value in pairs(gunSightNextProperties) do
        printLog( tostring(key) .. " : " .. tostring(value))
    end

    for styleValue, imageView in pairs(gunSightStyleIvList) do

        imageView:setEnabled(isEnable)

        local isSame = (styleValue == gunSightNextProperties[GUN_SIGHT_PROPERTY_STYLE])
        if isEnable and isSame then
            printLog("设置为选中态")
            imageView:setBackgroundImage(GUN_SIGHT_SELECTED_BG_IMAGE_PATH_TABLE[GUN_SIGHT_PROPERTY_STYLE])
        else
            printLog("设置为非选中态")
            imageView:setBackgroundColor("#00000000")
        end
    end

    for colorValue, imageView in pairs(gunSightColorIvList) do
        imageView:setEnabled(isEnable)
        if isEnable and colorValue == gunSightNextProperties[GUN_SIGHT_PROPERTY_COLOR] then
            imageView:setBackgroundImage(GUN_SIGHT_SELECTED_BG_IMAGE_PATH_TABLE[GUN_SIGHT_PROPERTY_COLOR])
        else
            imageView:setBackgroundColor("#00000000")
        end
    end

    for sizeValue, checkbox in pairs(gunSightSizeCbList) do
        checkbox:setEnabled(isEnable)
        if isEnable and sizeValue == gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE] then
            checkbox:setChecked(true)
        else
            checkbox:setChecked(false)
        end
    end
end

local function onCheckboxSelected(checkBoxId, isChecked)
    printLog("onCheckboxSelected : " .. tostring(checkBoxId) .. " : " .. tostring(isChecked))

    if checkBoxId == switchCheckBoxView:getId() then
        printLog("总开关点击事件：" .. tostring(isChecked))

        isCbSwitchChecked = isChecked
        commonlib_setSharedPrefBoolean(KEY_IS_SWITCH_CHECKED, isCbSwitchChecked)
        setWidgetEnable(isChecked)

        onSettingsChange(appName, DISPLAY_OPEN_GUN_SIGHT, isChecked)

    elseif checkBoxId == alwaysOpenCheckBoxView:getId() then
        isCbAlwaysOpenChecked = isChecked
        commonlib_setSharedPrefBoolean(KEY_IS_ALWAYS_OPEN, isCbAlwaysOpenChecked)
        if (gunSight ~= nil) then
            if isCbAlwaysOpenChecked then
                gunSight:show()
                showGunSightImpl(true);
            else
                showGunSightImpl(false);
            end
        end
        onSettingsChange(appName, DISPLAY_ALWAYS_OPEN_GUN_SIGHT, isChecked)
    end
end

-- 生成中间的标题栏，会返回标题栏左侧小圆点的ImageView 对象
local function generateSubTitle(tabPage, title, topView)
    local params = create("RelativeLayoutParam");
    local paramProperties = {
        layout_marginTop = "10dp",
        layout_marginLeft = "10dp",
        layout_width = "30dp",
        layout_height = "30dp",
        layout_below = topView:getId()
    }
    params:setLayoutParam(paramProperties);
    local dotImage = tabPage:addImageView(SETTING_VIEW_DOT_PATH, params, null);

    paramProperties = {
        layout_marginTop = "15dp",
        layout_marginLeft = "35dp",
        layout_width = "wrap_content",
        layout_height = "30dp",
        layout_alignLeft = dotImage:getId(),
        layout_below = topView:getId()
    };
    params:setLayoutParam(paramProperties);
    tabPage:addTextView(title, "#ffffff", "14sp", params, null);

    return dotImage;
end

-- 生成图标选择器
local function generateIconSelector(tabPage, iconPathList, topView, callback)
    local imageViewList = {}

    local lastImageView = nil
    for index, path in pairs(iconPathList) do
        local imageView = nil

        local params = create("RelativeLayoutParam");
        local paramProperties = {};

        paramProperties.layout_marginTop = "10dp"
        paramProperties.layout_width = "40dp"
        paramProperties.layout_height = "40dp"
        paramProperties.layout_below = topView:getId()

        if lastImageView == nil then
            paramProperties.layout_marginLeft = "40dp"
        else
            paramProperties.layout_marginLeft = "60dp"
            paramProperties.layout_alignLeft = lastImageView:getId()
        end

        params:setLayoutParam(paramProperties);
        imageView = tabPage:addImageView(path, params, callback)
        imageView:setPadding("3dp")
        imageViewList[index] = imageView;
        lastImageView = imageView;
    end

    return imageViewList;
end

-- 生成Checkbox 列表选择器
local function generateCheckboxSelector(tabPage, checkboxTextList, topView, callback)
    local checkboxList = {}

    local lastCheckbox = nil;
    for index, checkboxText in pairs(checkboxTextList) do
        local checkbox = nil;

        local params = create("RelativeLayoutParam");
        local paramProperties = {};
        paramProperties.layout_marginTop = "10dp"
        paramProperties.layout_marginBottom = "20dp"
        paramProperties.layout_below = topView:getId()

        if lastCheckbox == nil then
            paramProperties.layout_marginLeft = "40dp"
        else
            paramProperties.layout_marginLeft = "80dp"
            paramProperties.layout_alignLeft = lastCheckbox:getId()
        end
        params:setLayoutParam(paramProperties);
        checkbox = tabPage:addCheckBoxView(checkboxText, true, params, callback);

        lastCheckbox = checkbox;
        checkboxList[index] = checkbox;
    end

    return checkboxList
end

local function shouldUpdateGunSight()
    if gunSight == nil then
        return true;
    end

    if gunSightProperties[GUN_SIGHT_PROPERTY_STYLE] ~= gunSightNextProperties[GUN_SIGHT_PROPERTY_STYLE] then
        return true;
    end

    if gunSightProperties[GUN_SIGHT_PROPERTY_COLOR] ~= gunSightNextProperties[GUN_SIGHT_PROPERTY_COLOR] then
        return true;
    end

    if gunSightProperties[GUN_SIGHT_PROPERTY_SIZE] ~= gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE] then
        return true;
    end

    return false;
end

local function updateGunSight()
    printLog(" updateGunSight ")

    if not shouldUpdateGunSight() then
        return;
    end

    local item = GUN_SIGHT_POSITION

    if gunSight ~= nil then
        gunSight:destroy()
        gunSight = nil
    end

    local gunSightImgPath = GUN_SIGHT_IMG_PATH_DIR
            .. gunSightNextProperties[GUN_SIGHT_PROPERTY_STYLE]
            .. "/" .. gunSightNextProperties[GUN_SIGHT_PROPERTY_COLOR]
            .. ".png";

    printLog("gunSightImgPath : " .. tostring(gunSightImgPath))

    gunSight = floatView:addImageView(gunSightImgPath,
            item.rect.flag,
            item.rect.left,
            item.rect.top,
            gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE],
            gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE],
            true,
            ZOrder().zorder_bottom)
    gunSightProperties[GUN_SIGHT_PROPERTY_STYLE] = gunSightNextProperties[GUN_SIGHT_PROPERTY_STYLE];
    gunSightProperties[GUN_SIGHT_PROPERTY_COLOR] = gunSightNextProperties[GUN_SIGHT_PROPERTY_COLOR];
    gunSightProperties[GUN_SIGHT_PROPERTY_SIZE] = gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE];
    -- addImageView默认show
    gunSightShowing = true;
end

local function setSelectedImageView(selectedIvId, gunSightProperty)
    printLog("开始设置选中状态：" .. tostring(selectedIvId) .. " : " .. tostring(gunSightProperty))
    local selectedBgImagePath = GUN_SIGHT_SELECTED_BG_IMAGE_PATH_TABLE[gunSightProperty];
    printLog("selectedBgImagePath : " .. tostring(selectedBgImagePath))

    local ivList = nil
    if gunSightProperty == GUN_SIGHT_PROPERTY_STYLE then
        ivList = gunSightStyleIvList
    else
        ivList = gunSightColorIvList
    end

    for index, imageView in pairs(ivList) do
        printLog("current imageView Id: " .. tostring(imageView:getId()))
        if imageView:getId() == selectedIvId then
            printLog("设置为选中状态")
            imageView:setBackgroundImage(selectedBgImagePath)
            gunSightNextProperties[gunSightProperty] = index

            if gunSightProperty == GUN_SIGHT_PROPERTY_STYLE then
                commonlib_setSharedPrefString(KEY_GUN_SIGHT_STYLE, index)
            elseif gunSightProperty == GUN_SIGHT_PROPERTY_COLOR then
                commonlib_setSharedPrefString(KEY_GUN_SIGHT_COLOR, index)
            end
        else
            printLog("设置为非选中状态")
            imageView:setBackgroundColor("#00000000")
        end
    end
end

local function setSelectedCheckbox(checkboxId)
    printLog("setSelectedCheckbox : " .. checkboxId)
    for index, checkbox in pairs(gunSightSizeCbList) do
        printLog("current checkbox Id : " .. tostring(checkbox:getId()))
        if checkbox:getId() == checkboxId then
            printLog("设为选中状态")
            checkbox:setChecked(true)
            checkbox:setEnabled(false)
            gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE] = index
            commonlib_setSharedPrefInt(KEY_GUN_SIGHT_SIZE, index)
        else
            printLog("设为非选中状态")
            checkbox:setChecked(false)
            checkbox:setEnabled(true)
        end
    end
end

local function onGunSightStyleCallback(imageViewId)
    printLog("onGunSightStyleCallback : " .. imageViewId)
    setSelectedImageView(imageViewId, GUN_SIGHT_PROPERTY_STYLE)
end

local function onGunSightColorCallback(imageViewId)
    printLog("onGunSightColorCallback : " .. imageViewId)
    setSelectedImageView(imageViewId, GUN_SIGHT_PROPERTY_COLOR)
end

local function onGunSightSizeCallback(checkboxId, isChecked)
    printLog("onGunSightSizeCallback : " .. checkboxId)
    if isChecked then
        setSelectedCheckbox(checkboxId)
    end
end

local function initSettingUI()
    printLog(" initSettingUI ")

    local settingView = SettingView()
    local tabPage = settingView:addTableView(TAB_PAGE_NAME)

    local params = create("RelativeLayoutParam");
    local paramProperties;

    paramProperties = {
        layout_marginTop = "20dp",
        layout_marginLeft = "25dp"
    };
    params:setLayoutParam(paramProperties);
    isCbSwitchChecked = commonlib_getSharedPrefBoolean(KEY_IS_SWITCH_CHECKED, true)
    switchCheckBoxView = tabPage:addCheckBoxView(SWITCH_CHECKBOX_TEXT, isCbSwitchChecked, params, onCheckboxSelected);
    if isCbSwitchChecked then
        report_gamemaster_game_pubgm_state(9, 1)
    else
        report_gamemaster_game_pubgm_state(9, 2)
    end
    paramProperties = {
        layout_marginTop = "10dp",
        layout_marginLeft = "35dp",
        layout_below = switchCheckBoxView:getId()
    };
    params:setLayoutParam(paramProperties);
    alwaysOpenCheckBoxView = tabPage:addCheckBoxView(ALWAYS_OPEN_CHECKBOX_TEXT, true, params, onCheckboxSelected);

    local dotImageView = generateSubTitle(tabPage, "准星外观", alwaysOpenCheckBoxView)
    gunSightStyleIvList = generateIconSelector(tabPage, GUN_SIGHT_STYLE_ICON_LIST, dotImageView, onGunSightStyleCallback)

    dotImageView = generateSubTitle(tabPage, "准星颜色", gunSightStyleIvList[GUN_SIGHT_STYLE_ONE])
    gunSightColorIvList = generateIconSelector(tabPage, GUN_SIGHT_COLOR_ICON_LIST, dotImageView, onGunSightColorCallback)

    dotImageView = generateSubTitle(tabPage, "准星大小", gunSightColorIvList[GUN_SIGHT_COLOR_WHITE]);
    gunSightSizeCbList = generateCheckboxSelector(tabPage, GUN_SIGHT_SIZE_TEXT_LIST, dotImageView, onGunSightSizeCallback);

    gunSightNextProperties[GUN_SIGHT_PROPERTY_STYLE] = commonlib_getSharedPrefString(KEY_GUN_SIGHT_STYLE, GUN_SIGHT_STYLE_ONE);
    gunSightNextProperties[GUN_SIGHT_PROPERTY_COLOR] = commonlib_getSharedPrefString(KEY_GUN_SIGHT_COLOR, GUN_SIGHT_COLOR_WHITE);
    gunSightNextProperties[GUN_SIGHT_PROPERTY_SIZE] = commonlib_getSharedPrefInt(KEY_GUN_SIGHT_SIZE, GUN_SIGHT_IMAGE_SIZE_SMALL);

    isCbAlwaysOpenChecked = commonlib_getSharedPrefBoolean(KEY_IS_ALWAYS_OPEN, false)
    isCbAlwaysOpenLastChecked = isCbAlwaysOpenChecked;

    printLog("isCbAlwaysOpenChecked : " .. tostring(isCbAlwaysOpenChecked))
    printLog("isCbAlwaysOpenLastChecked : " .. tostring(isCbAlwaysOpenLastChecked))

    setWidgetEnable(isCbSwitchChecked)
end

local function onSettingViewOpen()
    gbSettingViewOpen = true
end

local function onSettingViewClose()
    gbSettingViewOpen = false
    commonlib_runOnUIThread(function ()
        local gameState = commonlib_getGameParam(KEY_GAME_STATE);
        printLog("the setting view is closed ! state = " .. tostring(gameState))
        if gameState == GAME_STATE_RUNNING then
            updateGunSight()
            if isCbSwitchChecked then
                showGunSightImpl(true)
            else
                showGunSightImpl(false)
            end
        end
    end)
    pressAllGameSettingsReport()
end

local function registerGameEvents()
    commonlib_registerSettingViewOpenCallback(onSettingViewOpen)
    commonlib_registerSettingViewCloseCallback(onSettingViewClose)
end

local function unregisterGameEvents()
    commonlib_unRegisterSettingViewOpenCallback(onSettingViewOpen)
    commonlib_unRegisterSettingViewCloseCallback(onSettingViewClose)
end

local function initVariable()
    GAME_ID = commonlib_getGameId()
    if GAME_ID == "com.tencent.tmgp.pubgmhd" then
        KEY_IS_SWITCH_CHECKED = "keypubgmhdIsSwitchChecked"--瞄准器
        KEY_IS_ALWAYS_OPEN = "keypubgmhdIsAlwaysOpen"
        KEY_IS_LAST_ALWAYS_OPEN = "keypubgmhdIsLastAlwaysOpen"
        KEY_GUN_SIGHT_STYLE = "keypubgmhdGunSightStyle"
        KEY_GUN_SIGHT_COLOR = "keypubgmhdGunSightColor"
        KEY_GUN_SIGHT_SIZE = "keypubgmhdGunSightSize"
    elseif GAME_ID == "com.tencent.tmgp.cf" then
        KEY_IS_SWITCH_CHECKED = "keyCfIsSwitchChecked"
        KEY_IS_ALWAYS_OPEN = "KeyCfIsAlwaysOpen"
        KEY_IS_LAST_ALWAYS_OPEN = "KeyCfIsLastAlwaysOpen"
        KEY_GUN_SIGHT_STYLE = "keyCfGunSightStyle"
        KEY_GUN_SIGHT_COLOR = "keyCfGunSightColor"
        KEY_GUN_SIGHT_SIZE = "keyCfGunSightSize"
    elseif GAME_ID == "com.tencent.cldts" then
        KEY_IS_SWITCH_CHECKED = "keyCldtsIsSwitchChecked"
        KEY_IS_ALWAYS_OPEN = "keyCldtsIsAlwaysOpen"
        KEY_IS_LAST_ALWAYS_OPEN = "keyCldtsIsLastAlwaysOpen"
        KEY_GUN_SIGHT_STYLE = "keyCldtsGunSightStyle"
        KEY_GUN_SIGHT_COLOR = "keyCldtsGunSightColor"
        KEY_GUN_SIGHT_SIZE = "keyCldtsGunSightSize"
    elseif GAME_ID == "com.netease.hyxd" then
        KEY_IS_SWITCH_CHECKED = "keyHyxdIsSwitchChecked"
        KEY_IS_ALWAYS_OPEN = "keyHyxdIsAlwaysOpen"
        KEY_IS_LAST_ALWAYS_OPEN = "keyHyxdIsLastAlwaysOpen"
        KEY_GUN_SIGHT_STYLE = "keyHyxdGunSightStyle"
        KEY_GUN_SIGHT_COLOR = "keyHyxdGunSightColor"
        KEY_GUN_SIGHT_SIZE = "keyHyxdGunSightSize"
    else
        KEY_IS_SWITCH_CHECKED = GAME_ID .. "KeyIsSwitchChecked"
        KEY_IS_ALWAYS_OPEN = GAME_ID .. "KeyIsAlwaysOpen"
        KEY_IS_LAST_ALWAYS_OPEN = GAME_ID .. "KeyIsLastAlwaysOpen"
        KEY_GUN_SIGHT_STYLE = GAME_ID .. "KeyGunSightStyle"
        KEY_GUN_SIGHT_COLOR = GAME_ID .. "KeyGunSightColor"
        KEY_GUN_SIGHT_SIZE = GAME_ID .. "KeyGunSightSize"
    end
    printLog("GAME_ID: " .. GAME_ID
            .. ", KEY_IS_SWITCH_CHECKED=" .. KEY_IS_SWITCH_CHECKED
            .. ", KEY_IS_ALWAYS_OPEN=" .. KEY_IS_ALWAYS_OPEN
            .. ", KEY_IS_LAST_ALWAYS_OPEN=" .. KEY_IS_LAST_ALWAYS_OPEN
            .. ", KEY_GUN_SIGHT_STYLE=" .. KEY_GUN_SIGHT_STYLE
            .. ", KEY_GUN_SIGHT_COLOR=" .. KEY_GUN_SIGHT_COLOR
            .. ", KEY_GUN_SIGHT_SIZE=" .. KEY_GUN_SIGHT_SIZE)
    floatView = commonlib_getUnClickableFloatView();
end

function common_gun_sight_init()
    printLog("init start")
    initVariable()
    registerGameEvents()
    initSettingUI()
    updateGunSight()
    printLog("init end")
end

function common_gun_sight_uninit()
    if gunSight ~= nil then
        gunSight:destroy();
        gunSight = nil;
    end
    unregisterGameEvents()
end

function common_gun_sight_getCbSwitchCheckState()
    return isCbSwitchChecked
end

function common_gun_sight_getCbAlwaysOpenCheckState()
    return isCbAlwaysOpenChecked
end

function common_gun_sight_getSettingViewOpenState()
    return gbSettingViewOpen
end

function common_gun_sight_show(isShowing)
    if isShowing then
        showGunSightImpl(true)
    else
        showGunSightImpl(false)
    end
end

function common_gun_sight_showOnUiThread(isShowing)
    if isShowing then
        showGunSightOnUiThread(true)
    else
        showGunSightOnUiThread(false)
    end
end

function common_gun_sight_updateGunSight()
    updateGunSight()
end