--- 这个脚本专门用于游戏设置的上报
--- 为避免单次打开面板操作同一个设置多次产生多条上报数据，所有上报先缓存，在用户关闭设置面板时才批量执行上报
--- 单个设置项点击两次（即：恢复原样），第一次是执行 table insert 操作，第二次执行的是 table remove

--
--      字段名        	      含义
-- |--------------|----------------------------|
-- game_name	      游戏类型：1：王者荣耀，2：荒野行动，3：和平精英，4：CF
-- settings_area      发生修改部位的值，1：战斗宝典开关，2：界面美化开关，3：界面美化皮肤种类，
--                                  5：显示红蓝buff倒计时，6：战斗指示开关，7：发生团战的时候，8：敌人靠近，
--                                  9：队友需要支援，10：敌人消失，11：经济总览开关，12：瞄准器开关，
--                                  13：始终显示瞄准器开关，14：准星外观，15：准星颜色，16：界面美化开关
--                                  20: 资源地图 28：跑圈提示
-- value_befor	      修改前的值，1：开关打开，2：开关关闭等
-- value_after	      修改后的值，1：开关打开，2：开关关闭等
--

local SWITCH_ON = 1
local SWITCH_OFF = 2

SGAME_NAME = 1
HYXD_NAME = 2
PUBGMHD_NAME = 3
CROSSFIRE_NAME = 4

DISPLAY_BATTLE_SKILL = 1
DISPLAY_SKIN_BEAUTIFY = 2
DISPLAY_BUFF_COUNTDOWN = 5

DISPLAY_BATTLE_ON_RUN_REMIND = 6
DISPLAY_BATTLE_CROWED_REMIND = 7
DISPLAY_ENEMY_CLOSING_REMIND = 8
DISPLAY_ALLY_SUFFERING_REMIND = 9
DISPLAY_ENEMIES_DISAPPEARED_REMIND = 10

DISPLAY_MONEY_COMPARISON = 11

DISPLAY_OPEN_GUN_SIGHT = 12
DISPLAY_ALWAYS_OPEN_GUN_SIGHT = 13
DISPLAY_OPEN_REPLAY = 17
DISPLAY_OPEN_RADAR = 19
DISPLAY_OPEN_MAP_RESOURCE = 20
DISPLAY_OPEN_HONOUR = 21--荣耀时刻开关
DISPLAY_OPEN_HONOUR_3_KILL = 22--3杀开关
DISPLAY_OPEN_HONOUR_4_KILL = 23--4杀开关
DISPLAY_OPEN_HONOUR_5_KILL = 24--5杀开关
DISPLAY_OPEN_HONOUR_RESTORE = 25 --荣耀时刻恢复默认设置
DISPLAY_OPEN_STIMULATE_MOMENT = 26 --刺激时刻开关
DISPLAY_OPEN_SETTING_KEYBOARD = 27 -- 超人键盘开关
DISPLAY_RUN_CIRCLE = 28 -- 跑圈提示开关
DISPLAY_CF_MOMENT = 29 --cf火线时刻开关

--gamemaster_settings_adjustment:374 game_name:byte settings_area:byte value_befor:byte value_after:byte uptime2:int network:byte pkgname:string
local TABLE_NAME = "gamemaster_settings_adjustment"

local function printGReportLog(msg)
    commonlib_print(TABLE_NAME, msg)
end

-- 用于缓存上报数据，以设置项作为 key
local report_entries = {}

function onSettingsChange(gameName, item, wasCheckedAfter)
    printGReportLog("onSettingsChange game[" .. tostring(gameName) .. "][" .. tostring(item) .. "] " .. tostring(wasCheckedAfter))

    -- 重复设置的场景执行移除操作
    if report_entries[item] ~= nil then
        report_entries[item] = nil
        return
    end

    report_entries[item] = function()
        local reportLib = Report()
        reportLib:report({
            table_name = TABLE_NAME,
            report_data = {
                game_name = gameName,
                settings_area = item,
                -- 三目运算
                value_befor = wasCheckedAfter and SWITCH_OFF or SWITCH_ON,
                value_after = wasCheckedAfter and SWITCH_ON or SWITCH_OFF,
                pkgname = commonlib_getGameId()
            }
        })
    end
end

function pressAllGameSettingsReport()
    printGReportLog("pressAllGameSettingsReport")

    -- 战斗提醒是联动设置，当总开关切换时，将子开关的上报移除，这里执行数据整理
    if report_entries[DISPLAY_BATTLE_ON_RUN_REMIND] ~= nil then
        printGReportLog("battle remind head switch presented, clear all sub switches")
        report_entries[DISPLAY_BATTLE_CROWED_REMIND] = nil
        report_entries[DISPLAY_ENEMY_CLOSING_REMIND] = nil
        report_entries[DISPLAY_ALLY_SUFFERING_REMIND] = nil
        report_entries[DISPLAY_ENEMIES_DISAPPEARED_REMIND] = nil
    end

    -- 瞄准器开关是联动设置，当总开关切换时，将子开关的上报移除，这里执行数据整理
    if report_entries[DISPLAY_OPEN_GUN_SIGHT] ~= nil then
        printGReportLog("gun sight head switch presented, clear all sub switches")
        report_entries[DISPLAY_ALWAYS_OPEN_GUN_SIGHT] = nil
    end

    for key, report in pairs(report_entries) do
        printGReportLog("report " .. tostring(key) .. " " .. tostring(report))
        report()
    end

    -- clear all entries
    report_entries = {}

    printGReportLog("pressAllGameSettingsReport finish")
end
