--
-- Created by IntelliJ IDEA.
-- User: hexujun
-- Date: 2018/6/21
-- Time: 20:28
-- To change this template use File | Settings | File Templates.
--

CLEAR_FLAG_NORMAL = "normal"
CLEAR_FLAG_FORCE = "force"
CLEAR_FLAG_CONSTANT = "constant"
CLEAR_FLAG_ALL =  "all"
CLEAR_FLAG_UNTIL_TRUE = "until_true"
CLEAR_FLAG_TIME_LIMIT_UNTIL_TRUE = "time_limit_until_true"
CLEAR_FLAG_LIMIT_TIME_EXIT_INCORRECTLY = "limit_time_exit_incorrectly"

local REPORT_INTERVAL = 1200 * 1000
local reportFunctionNameTable = {}

local function printLog(msg)
    commonlib_print("new_game_engine", msg);
end

--gamemaster_plugin_function:552 gameid:string functionname:string startcallcount:int endcallcount:int
local function report_gamemaster_plugin_function(functionname, startcallcount, endcallcount)
    commonlib_report("gamemaster_plugin_function",
            {
                ["startcallcount"] = startcallcount,
                ["endcallcount"]  = endcallcount,
                ["functionname"] = functionname,
                ["gameid"] = commonlib_getGameId(),
            });

    printLog("report_gamemaster_plugin_function, startcallcount:" .. tostring(startcallcount)
            .. ", endcallcount:" .. tostring(endcallcount)
            .. ", functionname:" .. tostring(functionname)
            .. ", gameId:" .. tostring(commonlib_getGameId()))
end

local function report_dispatch_gameEngine_function(functionName, callStart, callEnd, lastReportResult)
    if functionName == nil or type(functionName) ~= "string" then
        return
    end

    if reportFunctionNameTable[functionName] == nil then
        reportFunctionNameTable[functionName] = { callStartCount = 0, callEndCount = 0, last_report_time = 0}
    end

    local last_report_time =  reportFunctionNameTable[functionName].last_report_time
    local currentTime = commonlib_getCurrentTime()
    local diffTime = currentTime - last_report_time
    if last_report_time == 0 or diffTime >= REPORT_INTERVAL then
        if callStart ~= nil and callStart == true then
            reportFunctionNameTable[functionName].callStartCount = 1 + reportFunctionNameTable[functionName].callStartCount
            reportFunctionNameTable[functionName].last_report_time = currentTime
            report_gamemaster_plugin_function(functionName, reportFunctionNameTable[functionName].callStartCount, 0)
            return true
        end
    end

    if lastReportResult == true then
        if callEnd ~= nil and callEnd == true then
            reportFunctionNameTable[functionName].callEndCount = 1 + reportFunctionNameTable[functionName].callEndCount
            report_gamemaster_plugin_function(functionName, 0, reportFunctionNameTable[functionName].callEndCount)
            return true
        end
    end

    return false
end

CNewGameEngine = {
}

function CNewGameEngine:new()
    local object = {
        nativeEngineHandle = commonlib_getSkinManager():getNewEngineHandle(),
    };

    setmetatable(object, self);
    self.__index = self;

    return object;
end

local newGameEngine_;
function CNewGameEngine:getInstance()
    if newGameEngine_ == nil then
        newGameEngine_ = CNewGameEngine:new();
    end
    return newGameEngine_;
end

function CNewGameEngine:getSkinCanCreateFragment(skinId)
    return self:dispatch_gameEngine_function("getSkinCanCreateFragment", {skinId})
end

function CNewGameEngine:getSkinElementsShowStatus(skinId, fragmentId)
    return self:dispatch_gameEngine_function("getSkinElementsShowStatus", {skinId, fragmentId})
end

function CNewGameEngine:getSkinFragmentElements(skinId, fragmentId)
    return self:dispatch_gameEngine_function("getSkinFragmentElements", {skinId, fragmentId})
end

function CNewGameEngine:getAllSkinData()
    return self:dispatch_gameEngine_function("getAllSkinData", {})
end

function CNewGameEngine:setCurrentSkinId(skinId)
    return self:dispatch_gameEngine_function("setCurrentSkinId", {skinId})
end

--- param as {CLEAR_FLAG_NORMAL, CLEAR_FLAG_ALL}, {CLEAR_FLAG_NORMAL}
function CNewGameEngine:clearCache(clearFlags)
    if clearFlags == nil then
        return self:dispatch_gameEngine_function("clearCache", {})
    end

    return self:dispatch_gameEngine_function("clearCache", {clearFlags})
end

function CNewGameEngine:clearConditionCache(conditionName)
    return self:dispatch_gameEngine_function("clearConditionCache", {conditionName})
end

function CNewGameEngine:clearCRectAdjustManager()
    return self:dispatch_gameEngine_function("clearConditionCache", {"KeyClearCRectAdjustManager"})
end

function CNewGameEngine:loadSkin(skinId)
    return self:dispatch_gameEngine_function("loadSkin", {skinId})
end

function CNewGameEngine:getConditionRect(conditionRectName)
    return self:dispatch_gameEngine_function("getConditionRect", {conditionRectName})
end

function CNewGameEngine:dispatch_gameEngine_function(functionName, param)
    local hasReportStart = report_dispatch_gameEngine_function(functionName, true, nil, false)
    local result = native_dispatch_gameEngine_warp_fun(self.nativeEngineHandle, functionName, param);
    report_dispatch_gameEngine_function(functionName, nil, true, hasReportStart)
    return result
end

function CNewGameEngine:isSkinFragmentCanShow(skinId, fragmentId)
    return self:dispatch_gameEngine_function("isSkinFragmentCanShow", {skinId, fragmentId});
end

function CNewGameEngine:getSystemTimeAsMicrosecond()
    return self:dispatch_gameEngine_function("getSystemTimeAsMicrosecond", {});
end

function CNewGameEngine:matchCondition(conditionName)
    return self:dispatch_gameEngine_function("matchCondition", {conditionName});
end

function CNewGameEngine:matchConditionOnResult(conditionName)
    return self:dispatch_gameEngine_function("matchConditionOnResult", {conditionName});
end

function CNewGameEngine:matchConditionOnResultInRect(conditionName, x, y, w, h)
    return self:dispatch_gameEngine_function("matchConditionOnResult", {conditionName, x,y,w,h});
end

function CNewGameEngine:matchConditionOnResultInLandlords(conditionName, cards)
    return self:dispatch_gameEngine_function("matchConditionOnResult", {conditionName, cards});
end

function CNewGameEngine:matchConditionOnNumber(conditionName)
    local value = self:matchConditionOnResult(conditionName);
    if value == nil then
        return nil;
    end

    if type(value) == 'number' then
        return value;
    end

    return nil;
end

function CNewGameEngine:matchConditionOnString(conditionName)
    local value = self:matchConditionOnResult(conditionName);
    if value == nil then
        return nil;
    end

    if type(value) == 'string' then
        return value;
    end

    return nil;
end


function CNewGameEngine:matchConditionOnTable(conditionName)
    local value = self:matchConditionOnResult(conditionName);
    if value == nil then
        return nil;
    end

    if type(value) == 'table' then
        return value;
    end

    return nil;
end