-- 框架脚本
--[[
-- 该脚本是java层调用的入口，相当于是其他业务lua脚本的framwork层，
-- 为其他业务脚本提供统一的初始化，反初始化，以及初始化界面配置的
-- 函数，其work函数在java线程里面每250ms调用一次。

-- 其他业务脚本可以通过注册回调的方式向该框架脚本注册回调函数，这个
-- 脚本会根据各种事件或者状态的发生或者改变，通过回调函数回调到各个
-- 业务脚本里面去。
]]

local INIT_INDEX = 1
local UNINIT_INDEX = 2
local WORK_INDEX = 3
local TAG = "script_base"

local function printLog(msg)
    commonlib_print(TAG, msg)
end

local function clearTables()
    if common_life_circle_callbacks ~= nil then
        common_life_circle_callbacks = nil
    end

    if common_game_events_callbacks ~= nil then
        common_game_events_callbacks = nil
    end

    if core_scripts ~= nil then
        core_scripts = nil
    end
end

function baseInit()
    printLog("baseInit begin core_scripts size: "..tostring(#core_scripts))
    if commonlib_isTableEmpty(core_scripts) then
        printLog("baseInit core_scripts is empty")
        return
    end

    for i = 1, #core_scripts do
        if core_scripts[i] ~= nil and core_scripts[i][INIT_INDEX] ~= nil then
            printLog("baseInit: " .. tostring(core_scripts[i][INIT_INDEX]))
            core_scripts[i][INIT_INDEX]()
        end
    end
end

function baseUninit()
    if commonlib_isTableEmpty(core_scripts) then
        printLog("baseUninit core_scripts is empty")
        return
    end

    for i = 1, #core_scripts do
        if core_scripts[i] ~= nil and core_scripts[i][UNINIT_INDEX] ~= nil then
            printLog("baseUninit: " .. tostring(core_scripts[i][UNINIT_INDEX]))
            core_scripts[i][UNINIT_INDEX]()
        end
    end

    clearTables()
end

--- 工作函数，由java调度，脚本正常工作时每250ms调用一次
function baseWork()
    if commonlib_isTableEmpty(core_scripts) then
        printLog("baseWork core_scripts is empty")
        return
    end

    for i = 1, #core_scripts do
        if core_scripts[i] ~= nil and core_scripts[i][WORK_INDEX] ~= nil then
            printLog("script_base baseWork: " .. tostring(core_scripts[i][WORK_INDEX]))
            core_scripts[i][WORK_INDEX]()
        end
    end
end
